=== modified file 'NEWS'
--- NEWS	2007-02-15 15:11:31 +0000
+++ NEWS	2007-02-17 02:36:32 +0000
@@ -54,6 +54,13 @@
 
   INTERNALS:
 
+    * Internally revision ids are now passed around as utf-8 bytestrings,
+      rather than treating them as Unicode strings. This has performance
+      benefits for Knits, since we no longer need to decode the revision
+      id for each line of content, nor for each entry in the index. This
+      will also help with the future dirstate format. File-ids should
+      receive the same treatment in the future. (John Arbash Meinel)
+
     * Reserved ids (any revision-id ending in a colon) are rejected by
       versionedfiles, repositories, branches, and working trees
       (Aaron Bentley)

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2007-02-14 20:28:01 +0000
+++ bzrlib/branch.py	2007-02-17 02:33:45 +0000
@@ -378,6 +378,7 @@
         """Given a revision id, return its revno"""
         if revision_id is None:
             return 0
+        revision_id = osutils.safe_revision_id(revision_id)
         history = self.revision_history()
         try:
             return history.index(revision_id) + 1
@@ -580,6 +581,7 @@
         """
         new_history = self.revision_history()
         if revision_id is not None:
+            revision_id = osutils.safe_revision_id(revision_id)
             try:
                 new_history = new_history[:new_history.index(revision_id) + 1]
             except ValueError:
@@ -1245,6 +1247,7 @@
     @needs_write_lock
     def append_revision(self, *revision_ids):
         """See Branch.append_revision."""
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         for revision_id in revision_ids:
             _mod_revision.check_not_reserved_id(revision_id)
             mutter("add {%s} to revision-history" % revision_id)
@@ -1257,12 +1260,13 @@
 
         This performs the actual writing to disk.
         It is intended to be called by BzrBranch5.set_revision_history."""
-        self.control_files.put_utf8(
+        self.control_files.put_bytes(
             'revision-history', '\n'.join(history))
 
     @needs_write_lock
     def set_revision_history(self, rev_history):
         """See Branch.set_revision_history."""
+        rev_history = [osutils.safe_revision_id(r) for r in rev_history]
         self._write_revision_history(rev_history)
         transaction = self.get_transaction()
         history = transaction.map.find_revision_history()
@@ -1282,13 +1286,14 @@
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
+        revision_id = osutils.safe_revision_id(revision_id)
         history = self._lefthand_history(revision_id)
         assert len(history) == revno, '%d != %d' % (len(history), revno)
         self.set_revision_history(history)
 
     def _gen_revision_history(self):
-        decode_utf8 = cache_utf8.decode
-        history = [decode_utf8(l.rstrip('\r\n')) for l in
+        get_cached_utf8 = cache_utf8.get_cached_utf8
+        history = [get_cached_utf8(l.rstrip('\r\n')) for l in
                 self.control_files.get('revision-history').readlines()]
         return history
 
@@ -1338,6 +1343,7 @@
         :param other_branch: The other branch that DivergedBranches should
             raise with respect to.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         self.set_revision_history(self._lefthand_history(revision_id,
             last_rev, other_branch))
 
@@ -1351,6 +1357,8 @@
                 if stop_revision is None:
                     # if there are no commits, we're done.
                     return
+            else:
+                stop_revision = osutils.safe_revision_id(stop_revision)
             # whats the current last revision, before we fetch [and change it
             # possibly]
             last_rev = self.last_revision()
@@ -1500,7 +1508,6 @@
                     raise bzrlib.errors.InvalidURL(url,
                         "Urls must be 7-bit ascii, "
                         "use bzrlib.urlutils.escape")
-                    
             url = urlutils.relative_url(self.base, url)
         self._set_parent_location(url)
 
@@ -1509,7 +1516,7 @@
             self.control_files._transport.delete('parent')
         else:
             assert isinstance(url, str)
-            self.control_files.put('parent', StringIO(url + '\n'))
+            self.control_files.put_bytes('parent', url + '\n')
 
     @deprecated_function(zero_nine)
     def tree_config(self):
@@ -1692,8 +1699,9 @@
 
     @needs_read_lock
     def last_revision_info(self):
-        revision_string = self.control_files.get_utf8('last-revision').read()
+        revision_string = self.control_files.get('last-revision').read()
         revno, revision_id = revision_string.rstrip('\n').split(' ', 1)
+        revision_id = cache_utf8.get_cached_utf8(revision_id)
         revno = int(revno)
         return revno, revision_id
 
@@ -1716,10 +1724,11 @@
         if revision_id is None:
             revision_id = 'null:'
         out_string = '%d %s\n' % (revno, revision_id)
-        self.control_files.put_utf8('last-revision', out_string)
+        self.control_files.put_bytes('last-revision', out_string)
 
     @needs_write_lock
     def set_last_revision_info(self, revno, revision_id):
+        revision_id = osutils.safe_revision_id(revision_id)
         if self._get_append_revisions_only():
             self._check_history_violation(revision_id)
         self._write_last_revision_info(revno, revision_id)
@@ -1761,6 +1770,7 @@
 
     @needs_write_lock
     def append_revision(self, *revision_ids):
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         if len(revision_ids) == 0:
             return
         prev_revno, prev_revision = self.last_revision_info()

=== modified file 'bzrlib/cache_utf8.py'
--- bzrlib/cache_utf8.py	2006-11-29 17:16:58 +0000
+++ bzrlib/cache_utf8.py	2007-02-10 02:48:43 +0000
@@ -61,7 +61,8 @@
     try:
         return _utf8_to_uni[utf8_str]
     except KeyError:
-        _utf8_to_uni[utf8_str] = unicode_str = _utf8_decode(utf8_str)[0]
+        unicode_str = _utf8_decode(utf8_str)[0]
+        _utf8_to_uni[utf8_str] = unicode_str
         _uni_to_utf8[unicode_str] = utf8_str
         return unicode_str
 
@@ -78,6 +79,29 @@
     return decode(encode(unicode_str))
 
 
+def get_cached_utf8(utf8_str):
+    """Return a cached version of the utf-8 string.
+
+    Get a cached version of this string (similar to intern()).
+    At present, this will be decoded to ensure it is a utf-8 string. In the
+    future this might change to simply caching the string.
+    """
+    return encode(decode(utf8_str))
+
+
+def get_cached_ascii(ascii_str,
+                     _uni_to_utf8=_unicode_to_utf8_map,
+                     _utf8_to_uni=_utf8_to_unicode_map):
+    """This is a string which is identical in utf-8 and unicode."""
+    # We don't need to do any encoding, but we want _utf8_to_uni to return a
+    # real Unicode string. Unicode and plain strings of this type will have the
+    # same hash, so we can just use it as the key in _uni_to_utf8, but we need
+    # the return value to be different in _utf8_to_uni
+    ascii_str = _uni_to_utf8.setdefault(ascii_str, ascii_str)
+    _utf8_to_uni.setdefault(ascii_str, unicode(ascii_str))
+    return ascii_str
+
+
 def clear_encoding_cache():
     """Clear the encoding and decoding caches"""
     _unicode_to_utf8_map.clear()

=== modified file 'bzrlib/generate_ids.py'
--- bzrlib/generate_ids.py	2006-11-02 22:48:49 +0000
+++ bzrlib/generate_ids.py	2007-02-10 16:54:29 +0000
@@ -114,6 +114,7 @@
     if timestamp is None:
         timestamp = time.time()
 
-    return '-'.join((user_or_email,
-                    osutils.compact_date(timestamp),
-                    unique_chunk))
+    rev_id = u'-'.join((user_or_email,
+                        osutils.compact_date(timestamp),
+                        unique_chunk))
+    return rev_id.encode('utf8')

=== modified file 'bzrlib/knit.py'
--- bzrlib/knit.py	2007-01-31 18:58:10 +0000
+++ bzrlib/knit.py	2007-02-13 20:33:57 +0000
@@ -73,6 +73,7 @@
 from bzrlib import (
     cache_utf8,
     errors,
+    osutils,
     patiencediff,
     progress,
     ui,
@@ -156,9 +157,9 @@
 class _KnitFactory(object):
     """Base factory for creating content objects."""
 
-    def make(self, lines, version):
+    def make(self, lines, version_id):
         num_lines = len(lines)
-        return KnitContent(zip([version] * num_lines, lines))
+        return KnitContent(zip([version_id] * num_lines, lines))
 
 
 class KnitAnnotateFactory(_KnitFactory):
@@ -166,7 +167,7 @@
 
     annotated = True
 
-    def parse_fulltext(self, content, version):
+    def parse_fulltext(self, content, version_id):
         """Convert fulltext to internal representation
 
         fulltext content is of the format
@@ -174,17 +175,17 @@
         internal representation is of the format:
         (revid, plaintext)
         """
-        decode_utf8 = cache_utf8.decode
-        lines = []
-        for line in content:
-            origin, text = line.split(' ', 1)
-            lines.append((decode_utf8(origin), text))
+        # TODO: jam 20070209 The tests expect this to be returned as tuples,
+        #       but the code itself doesn't really depend on that.
+        #       Figure out a way to not require the overhead of turning the
+        #       list back into tuples.
+        lines = [tuple(line.split(' ', 1)) for line in content]
         return KnitContent(lines)
 
     def parse_line_delta_iter(self, lines):
         return iter(self.parse_line_delta(lines))
 
-    def parse_line_delta(self, lines, version):
+    def parse_line_delta(self, lines, version_id):
         """Convert a line based delta into internal representation.
 
         line delta is in the form of:
@@ -194,19 +195,19 @@
         internal representation is
         (start, end, count, [1..count tuples (revid, newline)])
         """
-        decode_utf8 = cache_utf8.decode
         result = []
         lines = iter(lines)
         next = lines.next
+
+        cache = {}
+        def cache_and_return(line):
+            origin, text = line.split(' ', 1)
+            return cache.setdefault(origin, origin), text
+
         # walk through the lines parsing.
         for header in lines:
             start, end, count = [int(n) for n in header.split(',')]
-            contents = []
-            remaining = count
-            while remaining:
-                origin, text = next().split(' ', 1)
-                remaining -= 1
-                contents.append((decode_utf8(origin), text))
+            contents = [tuple(next().split(' ', 1)) for i in xrange(count)]
             result.append((start, end, count, contents))
         return result
 
@@ -234,19 +235,21 @@
 
         see parse_fulltext which this inverts.
         """
-        encode_utf8 = cache_utf8.encode
-        return ['%s %s' % (encode_utf8(o), t) for o, t in content._lines]
+        # TODO: jam 20070209 We only do the caching thing to make sure that
+        #       the origin is a valid utf-8 line, eventually we could remove it
+        return ['%s %s' % (o, t) for o, t in content._lines]
 
     def lower_line_delta(self, delta):
         """convert a delta into a serializable form.
 
         See parse_line_delta which this inverts.
         """
-        encode_utf8 = cache_utf8.encode
+        # TODO: jam 20070209 We only do the caching thing to make sure that
+        #       the origin is a valid utf-8 line, eventually we could remove it
         out = []
         for start, end, c, lines in delta:
             out.append('%d,%d,%d\n' % (start, end, c))
-            out.extend(encode_utf8(origin) + ' ' + text
+            out.extend(origin + ' ' + text
                        for origin, text in lines)
         return out
 
@@ -256,26 +259,26 @@
 
     annotated = False
 
-    def parse_fulltext(self, content, version):
+    def parse_fulltext(self, content, version_id):
         """This parses an unannotated fulltext.
 
         Note that this is not a noop - the internal representation
         has (versionid, line) - its just a constant versionid.
         """
-        return self.make(content, version)
+        return self.make(content, version_id)
 
-    def parse_line_delta_iter(self, lines, version):
+    def parse_line_delta_iter(self, lines, version_id):
         cur = 0
         num_lines = len(lines)
         while cur < num_lines:
             header = lines[cur]
             cur += 1
             start, end, c = [int(n) for n in header.split(',')]
-            yield start, end, c, zip([version] * c, lines[cur:cur+c])
+            yield start, end, c, zip([version_id] * c, lines[cur:cur+c])
             cur += c
 
-    def parse_line_delta(self, lines, version):
-        return list(self.parse_line_delta_iter(lines, version))
+    def parse_line_delta(self, lines, version_id):
+        return list(self.parse_line_delta_iter(lines, version_id))
 
     def get_fulltext_content(self, lines):
         """Extract just the content lines from a fulltext."""
@@ -500,7 +503,7 @@
         return KnitVersionedFile(name, transport, factory=self.factory,
                                  delta=self.delta, create=True)
     
-    def _fix_parents(self, version, new_parents):
+    def _fix_parents(self, version_id, new_parents):
         """Fix the parents list for version.
         
         This is done by appending a new version to the index
@@ -508,9 +511,9 @@
         the parents list must be a superset of the current
         list.
         """
-        current_values = self._index._cache[version]
+        current_values = self._index._cache[version_id]
         assert set(current_values[4]).difference(set(new_parents)) == set()
-        self._index.add_version(version,
+        self._index.add_version(version_id,
                                 current_values[1], 
                                 current_values[2],
                                 current_values[3],
@@ -518,6 +521,7 @@
 
     def get_delta(self, version_id):
         """Get a delta for constructing version from some other version."""
+        version_id = osutils.safe_revision_id(version_id)
         self.check_not_reserved_id(version_id)
         if not self.has_version(version_id):
             raise RevisionNotPresent(version_id, self.filename)
@@ -529,10 +533,9 @@
             parent = None
         data_pos, data_size = self._index.get_position(version_id)
         data, sha1 = self._data.read_records(((version_id, data_pos, data_size),))[version_id]
-        version_idx = self._index.lookup(version_id)
         noeol = 'no-eol' in self._index.get_options(version_id)
         if 'fulltext' == self._index.get_method(version_id):
-            new_content = self.factory.parse_fulltext(data, version_idx)
+            new_content = self.factory.parse_fulltext(data, version_id)
             if parent is not None:
                 reference_content = self._get_content(parent)
                 old_texts = reference_content.text()
@@ -542,7 +545,7 @@
             delta_seq = KnitSequenceMatcher(None, old_texts, new_texts)
             return parent, sha1, noeol, self._make_line_delta(delta_seq, new_content)
         else:
-            delta = self.factory.parse_line_delta(data, version_idx)
+            delta = self.factory.parse_line_delta(data, version_id)
             return parent, sha1, noeol, delta
         
     def get_graph_with_ghosts(self):
@@ -552,6 +555,7 @@
 
     def get_sha1(self, version_id):
         """See VersionedFile.get_sha1()."""
+        version_id = osutils.safe_revision_id(version_id)
         record_map = self._get_record_map([version_id])
         method, content, digest, next = record_map[version_id]
         return digest 
@@ -563,6 +567,7 @@
 
     def has_ghost(self, version_id):
         """True if there is a ghost reference in the file to version_id."""
+        version_id = osutils.safe_revision_id(version_id)
         # maybe we have it
         if self.has_version(version_id):
             return False
@@ -581,6 +586,7 @@
 
     def has_version(self, version_id):
         """See VersionedFile.has_version."""
+        version_id = osutils.safe_revision_id(version_id)
         return self._index.has_version(version_id)
 
     __contains__ = has_version
@@ -738,6 +744,7 @@
             # I/O and the time spend applying deltas.
             delta = self._check_should_delta(present_parents)
 
+        assert isinstance(version_id, str)
         lines = self.factory.make(lines, version_id)
         if delta or (self.factory.annotated and len(present_parents) > 0):
             # Merge annotations from parent texts if so is needed.
@@ -799,6 +806,7 @@
 
     def get_line_list(self, version_ids):
         """Return the texts of listed versions as a list of strings."""
+        version_ids = [osutils.safe_revision_id(v) for v in version_ids]
         for version_id in version_ids:
             self.check_not_reserved_id(version_id)
         text_map, content_map = self._get_content_maps(version_ids)
@@ -834,12 +842,11 @@
                 if component_id in content_map:
                     content = content_map[component_id]
                 else:
-                    version_idx = self._index.lookup(component_id)
                     if method == 'fulltext':
                         assert content is None
-                        content = self.factory.parse_fulltext(data, version_idx)
+                        content = self.factory.parse_fulltext(data, version_id)
                     elif method == 'line-delta':
-                        delta = self.factory.parse_line_delta(data, version_idx)
+                        delta = self.factory.parse_line_delta(data, version_id)
                         content = content.copy()
                         content._lines = self._apply_delta(content._lines, 
                                                            delta)
@@ -865,6 +872,8 @@
         """See VersionedFile.iter_lines_added_or_present_in_versions()."""
         if version_ids is None:
             version_ids = self.versions()
+        else:
+            version_ids = [osutils.safe_revision_id(v) for v in version_ids]
         if pb is None:
             pb = progress.DummyProgress()
         # we don't care about inclusions, the caller cares.
@@ -887,7 +896,6 @@
             enumerate(self._data.read_records_iter(version_id_records)):
             pb.update('Walking content.', version_idx, total)
             method = self._index.get_method(version_id)
-            version_idx = self._index.lookup(version_id)
 
             assert method in ('fulltext', 'line-delta')
             if method == 'fulltext':
@@ -907,6 +915,7 @@
 
     def annotate_iter(self, version_id):
         """See VersionedFile.annotate_iter."""
+        version_id = osutils.safe_revision_id(version_id)
         content = self._get_content(version_id)
         for origin, text in content.annotate_iter():
             yield origin, text
@@ -916,6 +925,7 @@
         # perf notes:
         # optimism counts!
         # 52554 calls in 1264 872 internal down from 3674
+        version_id = osutils.safe_revision_id(version_id)
         try:
             return self._index.get_parents(version_id)
         except KeyError:
@@ -923,6 +933,7 @@
 
     def get_parents_with_ghosts(self, version_id):
         """See VersionedFile.get_parents."""
+        version_id = osutils.safe_revision_id(version_id)
         try:
             return self._index.get_parents_with_ghosts(version_id)
         except KeyError:
@@ -934,6 +945,7 @@
             versions = [versions]
         if not versions:
             return []
+        versions = [osutils.safe_revision_id(v) for v in versions]
         return self._index.get_ancestry(versions)
 
     def get_ancestry_with_ghosts(self, versions):
@@ -942,6 +954,7 @@
             versions = [versions]
         if not versions:
             return []
+        versions = [osutils.safe_revision_id(v) for v in versions]
         return self._index.get_ancestry_with_ghosts(versions)
 
     #@deprecated_method(zero_eight)
@@ -967,6 +980,8 @@
 
     def plan_merge(self, ver_a, ver_b):
         """See VersionedFile.plan_merge."""
+        ver_a = osutils.safe_revision_id(ver_a)
+        ver_b = osutils.safe_revision_id(ver_b)
         ancestors_b = set(self.get_ancestry(ver_b))
         def status_a(revision, text):
             if revision in ancestors_b:
@@ -1150,7 +1165,6 @@
     def _load_data(self, fp):
         cache = self._cache
         history = self._history
-        decode_utf8 = cache_utf8.decode
 
         self.check_header(fp)
         # readlines reads the whole file at once:
@@ -1175,12 +1189,13 @@
             for value in rec[4:-1]:
                 if value[0] == '.':
                     # uncompressed reference
-                    parents.append(decode_utf8(value[1:]))
+                    parent_id = value[1:]
                 else:
-                    parents.append(history[int(value)])
+                    parent_id = history[int(value)]
+                parents.append(parent_id)
 
             version_id, options, pos, size = rec[:4]
-            version_id = decode_utf8(version_id)
+            version_id = version_id
 
             # See self._cache_version
             # only want the _history index to reference the 1st 
@@ -1256,7 +1271,6 @@
         return self._cache[version_id][5]
 
     def _version_list_to_index(self, versions):
-        encode_utf8 = cache_utf8.encode
         result_list = []
         cache = self._cache
         for version in versions:
@@ -1265,7 +1279,7 @@
                 result_list.append(str(cache[version][5]))
                 # -- end lookup () --
             else:
-                result_list.append('.' + encode_utf8(version))
+                result_list.append('.' + version)
         return ' '.join(result_list)
 
     def add_version(self, version_id, options, pos, size, parents):
@@ -1279,13 +1293,12 @@
                          (version_id, options, pos, size, parents).
         """
         lines = []
-        encode_utf8 = cache_utf8.encode
         orig_history = self._history[:]
         orig_cache = self._cache.copy()
 
         try:
             for version_id, options, pos, size, parents in versions:
-                line = "\n%s %s %s %s %s :" % (encode_utf8(version_id),
+                line = "\n%s %s %s %s %s :" % (version_id,
                                                ','.join(options),
                                                pos,
                                                size,
@@ -1399,13 +1412,13 @@
         sio = StringIO()
         data_file = GzipFile(None, mode='wb', fileobj=sio)
 
-        version_id_utf8 = cache_utf8.encode(version_id)
+        assert isinstance(version_id, str)
         data_file.writelines(chain(
-            ["version %s %d %s\n" % (version_id_utf8,
+            ["version %s %d %s\n" % (version_id,
                                      len(lines),
                                      digest)],
             lines,
-            ["end %s\n" % version_id_utf8]))
+            ["end %s\n" % version_id]))
         data_file.close()
         length= sio.tell()
 
@@ -1461,7 +1474,7 @@
         if len(rec) != 4:
             raise KnitCorrupt(self._filename,
                               'unexpected number of elements in record header')
-        if cache_utf8.decode(rec[1]) != version_id:
+        if rec[1] != version_id:
             raise KnitCorrupt(self._filename,
                               'unexpected version, wanted %r, got %r'
                               % (version_id, rec[1]))

=== modified file 'bzrlib/lockable_files.py'
--- bzrlib/lockable_files.py	2006-09-05 19:58:39 +0000
+++ bzrlib/lockable_files.py	2007-02-09 23:39:24 +0000
@@ -196,11 +196,21 @@
         self._transport.put_file(self._escape(path), file, mode=self._file_mode)
 
     @needs_write_lock
+    def put_bytes(self, path, a_string):
+        """Write a string of bytes.
+
+        :param path: The path to put the bytes, relative to the transport root.
+        :param string: A string object, whose exact bytes are to be copied.
+        """
+        self._transport.put_bytes(self._escape(path), a_string,
+                                  mode=self._file_mode)
+
+    @needs_write_lock
     def put_utf8(self, path, a_string):
         """Write a string, encoding as utf-8.
 
         :param path: The path to put the string, relative to the transport root.
-        :param string: A file-like or string object whose contents should be copied.
+        :param string: A string or unicode object whose contents should be copied.
         """
         # IterableFile would not be needed if Transport.put took iterables
         # instead of files.  ADHB 2005-12-25
@@ -210,7 +220,7 @@
         # these are valuable files which should have exact contents.
         if not isinstance(a_string, basestring):
             raise errors.BzrBadParameterNotString(a_string)
-        self.put(path, StringIO(a_string.encode('utf-8')))
+        self.put_bytes(path, a_string.encode('utf-8'))
 
     def lock_write(self):
         # mutter("lock write: %s (%s)", self, self._lock_count)

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-02-14 10:27:53 +0000
+++ bzrlib/osutils.py	2007-02-17 01:42:57 +0000
@@ -47,6 +47,7 @@
 import unicodedata
 
 from bzrlib import (
+    cache_utf8,
     errors,
     win32utils,
     )
@@ -813,9 +814,12 @@
     # 2) Isn't one of ' \t\r\n' which are characters we sometimes use as
     #    separators
     # 3) '\xa0' isn't unicode safe since it is >128.
-    # So we are following textwrap's example and hard-coding our own.
-    # We probably could ignore \v and \f, too.
-    for ch in u' \t\n\r\v\f':
+
+    # This should *not* be a unicode set of characters in case the source
+    # string is not a Unicode string. We can auto-up-cast the characters since
+    # they are ascii, but we don't want to auto-up-cast the string in case it
+    # is utf-8
+    for ch in ' \t\n\r\v\f':
         if ch in s:
             return True
     else:
@@ -881,6 +885,43 @@
         raise errors.BzrBadParameterNotUnicode(unicode_or_utf8_string)
 
 
+def safe_utf8(unicode_or_utf8_string):
+    """Coerce unicode_or_utf8_string to a utf8 string.
+
+    If it is a str, it is returned.
+    If it is Unicode, it is encoded into a utf-8 string.
+    """
+    if isinstance(unicode_or_utf8_string, str):
+        # TODO: jam 20070209 This is overkill, and probably has an impact on
+        #       performance if we are dealing with lots of apis that want a
+        #       utf-8 revision id
+        try:
+            # Make sure it is a valid utf-8 string
+            unicode_or_utf8_string.decode('utf-8')
+        except UnicodeDecodeError:
+            raise errors.BzrBadParameterNotUnicode(unicode_or_utf8_string)
+        return unicode_or_utf8_string
+    return unicode_or_utf8_string.encode('utf-8')
+
+
+def safe_revision_id(unicode_or_utf8_string):
+    """Revision ids should now be utf8, but at one point they were unicode.
+
+    This is the same as safe_utf8, except it uses the cached encode functions
+    to save a little bit of performance.
+    """
+    if unicode_or_utf8_string is None:
+        return None
+    if isinstance(unicode_or_utf8_string, str):
+        # TODO: jam 20070209 Eventually just remove this check.
+        try:
+            utf8_str = cache_utf8.get_cached_utf8(unicode_or_utf8_string)
+        except UnicodeDecodeError:
+            raise errors.BzrBadParameterNotUnicode(unicode_or_utf8_string)
+        return utf8_str
+    return cache_utf8.encode(unicode_or_utf8_string)
+
+
 _platform_normalizes_filenames = False
 if sys.platform == 'darwin':
     _platform_normalizes_filenames = True

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2007-02-14 20:28:01 +0000
+++ bzrlib/repofmt/knitrepo.py	2007-02-17 01:42:57 +0000
@@ -20,6 +20,7 @@
     knit,
     lockable_files,
     lockdir,
+    osutils,
     transactions,
     xml5,
     xml6,
@@ -63,6 +64,8 @@
         This determines the set of revisions which are involved, and then
         finds all file ids affected by those revisions.
         """
+        from_revid = osutils.safe_revision_id(from_revid)
+        to_revid = osutils.safe_revision_id(to_revid)
         vf = self._get_revision_vf()
         from_set = set(vf.get_ancestry(from_revid))
         to_set = set(vf.get_ancestry(to_revid))
@@ -90,6 +93,7 @@
         """
         if revision_id is None:
             return [None]
+        revision_id = osutils.safe_revision_id(revision_id)
         vf = self._get_revision_vf()
         try:
             return [None] + vf.get_ancestry(revision_id)
@@ -99,6 +103,7 @@
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
+        revision_id = osutils.safe_revision_id(revision_id)
         return self.get_revision_reconcile(revision_id)
 
     @needs_read_lock
@@ -113,6 +118,7 @@
         # special case NULL_REVISION
         if revision_id == _mod_revision.NULL_REVISION:
             return {}
+        revision_id = osutils.safe_revision_id(revision_id)
         a_weave = self._get_revision_vf()
         entire_graph = a_weave.get_graph()
         if revision_id is None:
@@ -145,7 +151,7 @@
             pending = set(self.all_revision_ids())
             required = set([])
         else:
-            pending = set(revision_ids)
+            pending = set(osutils.safe_revision_id(r) for r in revision_ids)
             # special case NULL_REVISION
             if _mod_revision.NULL_REVISION in pending:
                 pending.remove(_mod_revision.NULL_REVISION)
@@ -193,6 +199,7 @@
         return reconciler
     
     def revision_parents(self, revision_id):
+        revision_id = osutils.safe_revision_id(revision_id)
         return self._get_revision_vf().get_parents(revision_id)
 
 
@@ -238,6 +245,7 @@
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         return RootCommitBuilder(self, parents, config, timestamp, timezone,
                                  committer, revprops, revision_id)
 

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2007-02-14 20:28:01 +0000
+++ bzrlib/repository.py	2007-02-17 01:42:57 +0000
@@ -78,39 +78,41 @@
         )
 
     @needs_write_lock
-    def add_inventory(self, revid, inv, parents):
-        """Add the inventory inv to the repository as revid.
+    def add_inventory(self, revision_id, inv, parents):
+        """Add the inventory inv to the repository as revision_id.
         
-        :param parents: The revision ids of the parents that revid
+        :param parents: The revision ids of the parents that revision_id
                         is known to have and are in the repository already.
 
         returns the sha1 of the serialized inventory.
         """
-        _mod_revision.check_not_reserved_id(revid)
-        assert inv.revision_id is None or inv.revision_id == revid, \
+        revision_id = osutils.safe_revision_id(revision_id)
+        _mod_revision.check_not_reserved_id(revision_id)
+        assert inv.revision_id is None or inv.revision_id == revision_id, \
             "Mismatch between inventory revision" \
-            " id and insertion revid (%r, %r)" % (inv.revision_id, revid)
+            " id and insertion revid (%r, %r)" % (inv.revision_id, revision_id)
         assert inv.root is not None
         inv_text = self.serialise_inventory(inv)
         inv_sha1 = osutils.sha_string(inv_text)
         inv_vf = self.control_weaves.get_weave('inventory',
                                                self.get_transaction())
-        self._inventory_add_lines(inv_vf, revid, parents, osutils.split_lines(inv_text))
+        self._inventory_add_lines(inv_vf, revision_id, parents,
+                                  osutils.split_lines(inv_text))
         return inv_sha1
 
-    def _inventory_add_lines(self, inv_vf, revid, parents, lines):
+    def _inventory_add_lines(self, inv_vf, revision_id, parents, lines):
         final_parents = []
         for parent in parents:
             if parent in inv_vf:
                 final_parents.append(parent)
 
-        inv_vf.add_lines(revid, final_parents, lines)
+        inv_vf.add_lines(revision_id, final_parents, lines)
 
     @needs_write_lock
-    def add_revision(self, rev_id, rev, inv=None, config=None):
-        """Add rev to the revision store as rev_id.
+    def add_revision(self, revision_id, rev, inv=None, config=None):
+        """Add rev to the revision store as revision_id.
 
-        :param rev_id: the revision id to use.
+        :param revision_id: the revision id to use.
         :param rev: The revision object.
         :param inv: The inventory for the revision. if None, it will be looked
                     up in the inventory storer
@@ -118,20 +120,23 @@
                        If supplied its signature_needed method will be used
                        to determine if a signature should be made.
         """
-        _mod_revision.check_not_reserved_id(rev_id)
+        revision_id = osutils.safe_revision_id(revision_id)
+        # TODO: jam 20070210 Shouldn't we check rev.revision_id and
+        #       rev.parent_ids?
+        _mod_revision.check_not_reserved_id(revision_id)
         if config is not None and config.signature_needed():
             if inv is None:
-                inv = self.get_inventory(rev_id)
+                inv = self.get_inventory(revision_id)
             plaintext = Testament(rev, inv).as_short_text()
             self.store_revision_signature(
-                gpg.GPGStrategy(config), plaintext, rev_id)
-        if not rev_id in self.get_inventory_weave():
+                gpg.GPGStrategy(config), plaintext, revision_id)
+        if not revision_id in self.get_inventory_weave():
             if inv is None:
-                raise errors.WeaveRevisionNotPresent(rev_id,
+                raise errors.WeaveRevisionNotPresent(revision_id,
                                                      self.get_inventory_weave())
             else:
                 # yes, this is not suitable for adding with ghosts.
-                self.add_inventory(rev_id, inv, rev.parent_ids)
+                self.add_inventory(revision_id, inv, rev.parent_ids)
         self._revision_store.add_revision(rev, self.get_transaction())
 
     @needs_read_lock
@@ -159,6 +164,9 @@
         if self._revision_store.text_store.listable():
             return self._revision_store.all_revision_ids(self.get_transaction())
         result = self._all_possible_ids()
+        # TODO: jam 20070210 Ensure that _all_possible_ids returns non-unicode
+        #       ids. (It should, since _revision_store's API should change to
+        #       return utf8 revision_ids)
         return self._eliminate_revisions_not_present(result)
 
     def break_lock(self):
@@ -287,6 +295,7 @@
 
         revision_id: only return revision ids included by revision_id.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         return InterRepository.get(other, self).missing_revision_ids(revision_id)
 
     @staticmethod
@@ -305,6 +314,7 @@
         This is a destructive operation! Do not use it on existing 
         repositories.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         return InterRepository.get(self, destination).copy_content(revision_id, basis)
 
     def fetch(self, source, revision_id=None, pb=None):
@@ -312,6 +322,7 @@
 
         If revision_id is None all content is copied.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         return InterRepository.get(source, self).fetch(revision_id=revision_id,
                                                        pb=pb)
 
@@ -329,6 +340,7 @@
         :param revprops: Optional dictionary of revision properties.
         :param revision_id: Optional revision id.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         return _CommitBuilder(self, parents, config, timestamp, timezone,
                               committer, revprops, revision_id)
 
@@ -360,6 +372,7 @@
     @needs_read_lock
     def has_revision(self, revision_id):
         """True if this repository has a copy of the revision."""
+        revision_id = osutils.safe_revision_id(revision_id)
         return self._revision_store.has_revision_id(revision_id,
                                                     self.get_transaction())
 
@@ -375,16 +388,26 @@
         if not revision_id or not isinstance(revision_id, basestring):
             raise errors.InvalidRevisionId(revision_id=revision_id,
                                            branch=self)
-        return self._revision_store.get_revisions([revision_id],
-                                                  self.get_transaction())[0]
+        return self.get_revisions([revision_id])[0]
+
     @needs_read_lock
     def get_revisions(self, revision_ids):
-        return self._revision_store.get_revisions(revision_ids,
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
+        revs = self._revision_store.get_revisions(revision_ids,
                                                   self.get_transaction())
+        for rev in revs:
+            assert not isinstance(rev.revision_id, unicode)
+            for parent_id in rev.parent_ids:
+                assert not isinstance(parent_id, unicode)
+        return revs
 
     @needs_read_lock
     def get_revision_xml(self, revision_id):
-        rev = self.get_revision(revision_id) 
+        # TODO: jam 20070210 This shouldn't be necessary since get_revision
+        #       would have already do it.
+        # TODO: jam 20070210 Just use _serializer.write_revision_to_string()
+        revision_id = osutils.safe_revision_id(revision_id)
+        rev = self.get_revision(revision_id)
         rev_tmp = StringIO()
         # the current serializer..
         self._revision_store._serializer.write_revision(rev, rev_tmp)
@@ -394,6 +417,8 @@
     @needs_read_lock
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
+        # TODO: jam 20070210 get_revision_reconcile should do this for us
+        revision_id = osutils.safe_revision_id(revision_id)
         r = self.get_revision_reconcile(revision_id)
         # weave corruption can lead to absent revision markers that should be
         # present.
@@ -455,6 +480,7 @@
 
     @needs_write_lock
     def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        revision_id = osutils.safe_revision_id(revision_id)
         signature = gpg_strategy.sign(plaintext)
         self._revision_store.add_revision_signature_text(revision_id,
                                                          signature,
@@ -471,7 +497,8 @@
         assert self._serializer.support_altered_by_hack, \
             ("fileids_altered_by_revision_ids only supported for branches " 
              "which store inventory as unnested xml, not on %r" % self)
-        selected_revision_ids = set(revision_ids)
+        selected_revision_ids = set(osutils.safe_revision_id(r)
+                                    for r in revision_ids)
         w = self.get_inventory_weave()
         result = {}
 
@@ -541,6 +568,9 @@
     @needs_read_lock
     def get_inventory(self, revision_id):
         """Get Inventory object by hash."""
+        # TODO: jam 20070210 Technically we don't need to sanitize, since all
+        #       called functions must sanitize.
+        revision_id = osutils.safe_revision_id(revision_id)
         return self.deserialise_inventory(
             revision_id, self.get_inventory_xml(revision_id))
 
@@ -550,6 +580,7 @@
         :param revision_id: The expected revision id of the inventory.
         :param xml: A serialised inventory.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         result = self._serializer.read_inventory_from_string(xml)
         result.root.revision = revision_id
         return result
@@ -560,8 +591,9 @@
     @needs_read_lock
     def get_inventory_xml(self, revision_id):
         """Get inventory XML as a file object."""
+        revision_id = osutils.safe_revision_id(revision_id)
         try:
-            assert isinstance(revision_id, basestring), type(revision_id)
+            assert isinstance(revision_id, str), type(revision_id)
             iw = self.get_inventory_weave()
             return iw.get_text(revision_id)
         except IndexError:
@@ -571,6 +603,8 @@
     def get_inventory_sha1(self, revision_id):
         """Return the sha1 hash of the inventory entry
         """
+        # TODO: jam 20070210 Shouldn't this be deprecated / removed?
+        revision_id = osutils.safe_revision_id(revision_id)
         return self.get_revision(revision_id).inventory_sha1
 
     @needs_read_lock
@@ -585,6 +619,7 @@
         # special case NULL_REVISION
         if revision_id == _mod_revision.NULL_REVISION:
             return {}
+        revision_id = osutils.safe_revision_id(revision_id)
         a_weave = self.get_inventory_weave()
         all_revisions = self._eliminate_revisions_not_present(
                                 a_weave.versions())
@@ -618,7 +653,7 @@
             pending = set(self.all_revision_ids())
             required = set([])
         else:
-            pending = set(revision_ids)
+            pending = set(osutils.safe_revision_id(r) for r in revision_ids)
             # special case NULL_REVISION
             if _mod_revision.NULL_REVISION in pending:
                 pending.remove(_mod_revision.NULL_REVISION)
@@ -657,6 +692,7 @@
         :param revision_id: The revision id to start with.  All its lefthand
             ancestors will be traversed.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         if revision_id in (None, _mod_revision.NULL_REVISION):
             return
         next_id = revision_id
@@ -710,6 +746,7 @@
             return RevisionTree(self, Inventory(root_id=None), 
                                 _mod_revision.NULL_REVISION)
         else:
+            revision_id = osutils.safe_revision_id(revision_id)
             inv = self.get_revision_inventory(revision_id)
             return RevisionTree(self, inv, revision_id)
 
@@ -737,6 +774,7 @@
         """
         if revision_id is None:
             return [None]
+        revision_id = osutils.safe_revision_id(revision_id)
         if not self.has_revision(revision_id):
             raise errors.NoSuchRevision(self, revision_id)
         w = self.get_inventory_weave()
@@ -751,6 +789,7 @@
         - it writes to stdout, it assumes that that is valid etc. Fix
         by creating a new more flexible convenience function.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         tree = self.revision_tree(revision_id)
         # use inventory as it was in that revision
         file_id = tree.inventory.path2id(file)
@@ -764,8 +803,9 @@
     def get_transaction(self):
         return self.control_files.get_transaction()
 
-    def revision_parents(self, revid):
-        return self.get_inventory_weave().parent_names(revid)
+    def revision_parents(self, revision_id):
+        revision_id = osutils.safe_revision_id(revision_id)
+        return self.get_inventory_weave().parent_names(revision_id)
 
     @needs_write_lock
     def set_make_working_trees(self, new_value):
@@ -785,18 +825,21 @@
 
     @needs_write_lock
     def sign_revision(self, revision_id, gpg_strategy):
+        revision_id = osutils.safe_revision_id(revision_id)
         plaintext = Testament.from_revision(self, revision_id).as_short_text()
         self.store_revision_signature(gpg_strategy, plaintext, revision_id)
 
     @needs_read_lock
     def has_signature_for_revision_id(self, revision_id):
         """Query for a revision signature for revision_id in the repository."""
+        revision_id = osutils.safe_revision_id(revision_id)
         return self._revision_store.has_signature(revision_id,
                                                   self.get_transaction())
 
     @needs_read_lock
     def get_signature_text(self, revision_id):
         """Return the text for a signature."""
+        revision_id = osutils.safe_revision_id(revision_id)
         return self._revision_store.get_signature_text(revision_id,
                                                        self.get_transaction())
 
@@ -812,6 +855,7 @@
         if not revision_ids:
             raise ValueError("revision_ids must be non-empty in %s.check" 
                     % (self,))
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         return self._check(revision_ids)
 
     def _check(self, revision_ids):
@@ -840,6 +884,11 @@
                     revision_id.encode('ascii')
                 except UnicodeEncodeError:
                     raise errors.NonAsciiRevisionId(method, self)
+            else:
+                try:
+                    revision_id.decode('ascii')
+                except UnicodeDecodeError:
+                    raise errors.NonAsciiRevisionId(method, self)
 
 
 
@@ -1253,6 +1302,9 @@
         # generic, possibly worst case, slow code path.
         target_ids = set(self.target.all_revision_ids())
         if revision_id is not None:
+            # TODO: jam 20070210 InterRepository is internal enough that it
+            #       should assume revision_ids are already utf-8
+            revision_id = osutils.safe_revision_id(revision_id)
             source_ids = self.source.get_ancestry(revision_id)
             assert source_ids[0] is None
             source_ids.pop(0)
@@ -1302,6 +1354,9 @@
             self.target.set_make_working_trees(self.source.make_working_trees())
         except NotImplementedError:
             pass
+        # TODO: jam 20070210 This is fairly internal, so we should probably
+        #       just assert that revision_id is not unicode.
+        revision_id = osutils.safe_revision_id(revision_id)
         # grab the basis available data
         if basis is not None:
             self.target.fetch(basis, revision_id=revision_id)
@@ -1318,6 +1373,8 @@
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, 
                self.target._format)
+        # TODO: jam 20070210 This should be an assert, not a translate
+        revision_id = osutils.safe_revision_id(revision_id)
         f = GenericRepoFetcher(to_repository=self.target,
                                from_repository=self.source,
                                last_revision=revision_id,
@@ -1360,6 +1417,8 @@
     def copy_content(self, revision_id=None, basis=None):
         """See InterRepository.copy_content()."""
         # weave specific optimised path:
+        # TODO: jam 20070210 Internal, should be an assert, not translate
+        revision_id = osutils.safe_revision_id(revision_id)
         if basis is not None:
             # copy the basis in, then fetch remaining data.
             basis.copy_content_into(self.target, revision_id)
@@ -1402,6 +1461,8 @@
         from bzrlib.fetch import GenericRepoFetcher
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, self.target._format)
+        # TODO: jam 20070210 This should be an assert, not a translate
+        revision_id = osutils.safe_revision_id(revision_id)
         f = GenericRepoFetcher(to_repository=self.target,
                                from_repository=self.source,
                                last_revision=revision_id,
@@ -1479,6 +1540,8 @@
         from bzrlib.fetch import KnitRepoFetcher
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, self.target._format)
+        # TODO: jam 20070210 This should be an assert, not a translate
+        revision_id = osutils.safe_revision_id(revision_id)
         f = KnitRepoFetcher(to_repository=self.target,
                             from_repository=self.source,
                             last_revision=revision_id,
@@ -1537,6 +1600,8 @@
     def fetch(self, revision_id=None, pb=None):
         """See InterRepository.fetch()."""
         from bzrlib.fetch import Model1toKnit2Fetcher
+        # TODO: jam 20070210 This should be an assert, not a translate
+        revision_id = osutils.safe_revision_id(revision_id)
         f = Model1toKnit2Fetcher(to_repository=self.target,
                                  from_repository=self.source,
                                  last_revision=revision_id,
@@ -1558,6 +1623,8 @@
             self.target.set_make_working_trees(self.source.make_working_trees())
         except NotImplementedError:
             pass
+        # TODO: jam 20070210 Internal, assert, don't translate
+        revision_id = osutils.safe_revision_id(revision_id)
         # grab the basis available data
         if basis is not None:
             self.target.fetch(basis, revision_id=revision_id)
@@ -1593,6 +1660,8 @@
         mutter("Using fetch logic to copy between %s(%s) and %s(%s)",
                self.source, self.source._format, self.target, 
                self.target._format)
+        # TODO: jam 20070210 This should be an assert, not a translate
+        revision_id = osutils.safe_revision_id(revision_id)
         f = Knit1to2Fetcher(to_repository=self.target,
                             from_repository=self.source,
                             last_revision=revision_id,
@@ -1780,7 +1849,7 @@
             self._committer = committer
 
         self.new_inventory = Inventory(None)
-        self._new_revision_id = revision_id
+        self._new_revision_id = osutils.safe_revision_id(revision_id)
         self.parents = parents
         self.repository = repository
 

=== modified file 'bzrlib/revisiontree.py'
--- bzrlib/revisiontree.py	2006-12-17 17:42:43 +0000
+++ bzrlib/revisiontree.py	2007-02-10 16:54:29 +0000
@@ -18,6 +18,9 @@
 
 from cStringIO import StringIO
 
+from bzrlib import (
+    osutils,
+    )
 from bzrlib.tree import Tree
 
 
@@ -40,7 +43,7 @@
         self._repository = branch
         self._weave_store = branch.weave_store
         self._inventory = inv
-        self._revision_id = revision_id
+        self._revision_id = osutils.safe_revision_id(revision_id)
 
     def get_parent_ids(self):
         """See Tree.get_parent_ids.

=== modified file 'bzrlib/store/revision/__init__.py'
--- bzrlib/store/revision/__init__.py	2006-11-10 21:06:11 +0000
+++ bzrlib/store/revision/__init__.py	2007-02-10 02:48:43 +0000
@@ -28,12 +28,11 @@
 from copy import deepcopy
 import unittest
 
-import bzrlib
 from bzrlib import (
     errors,
+    osutils,
     xml5,
     )
-import bzrlib.store
 """)
 
 from bzrlib.trace import mutter
@@ -127,6 +126,7 @@
         
         :return: a signature text.
         """
+        revision_id = osutils.safe_revision_id(revision_id)
         self._guard_revision(revision_id, transaction)
         return self._get_signature_text(revision_id, transaction)
 
@@ -145,6 +145,7 @@
 
     def has_signature(self, revision_id, transaction):
         """True if the store has a signature for revision_id."""
+        revision_id = osutils.safe_revision_id(revision_id)
         self._guard_revision(revision_id, transaction)
         return self._has_signature(revision_id, transaction)
 

=== modified file 'bzrlib/store/revision/knit.py'
--- bzrlib/store/revision/knit.py	2006-12-12 15:26:42 +0000
+++ bzrlib/store/revision/knit.py	2007-02-10 02:48:43 +0000
@@ -21,7 +21,6 @@
 """
 
 
-import bzrlib
 from bzrlib import errors, osutils
 from bzrlib.knit import KnitVersionedFile, KnitPlainFactory
 from bzrlib.store.revision import RevisionStore
@@ -70,6 +69,7 @@
 
     def add_revision_signature_text(self, revision_id, signature_text, transaction):
         """See RevisionStore.add_revision_signature_text()."""
+        revision_id = osutils.safe_revision_id(revision_id)
         self.get_signature_file(transaction).add_lines(
             revision_id, [], osutils.split_lines(signature_text))
 
@@ -80,6 +80,7 @@
 
     def get_revisions(self, revision_ids, transaction):
         """See RevisionStore.get_revisions()."""
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         texts = self._get_serialized_revisions(revision_ids, transaction)
         revisions = []
         try:
@@ -90,7 +91,7 @@
         except SyntaxError, e:
             raise errors.BzrError('failed to unpack revision_xml for %s: %s' %
                                    (revision_id, str(e)))
-        return revisions 
+        return revisions
 
     def _get_serialized_revisions(self, revision_ids, transaction):
         texts = []
@@ -123,9 +124,10 @@
 
     def has_revision_id(self, revision_id, transaction):
         """True if the store contains revision_id."""
+        revision_id = osutils.safe_revision_id(revision_id)
         return (revision_id is None
                 or self.get_revision_file(transaction).has_version(revision_id))
-        
+
     def _has_signature(self, revision_id, transaction):
         """See RevisionStore._has_signature()."""
         return self.get_signature_file(transaction).has_version(revision_id)

=== modified file 'bzrlib/store/revision/text.py'
--- bzrlib/store/revision/text.py	2006-11-10 21:06:11 +0000
+++ bzrlib/store/revision/text.py	2007-02-10 02:48:43 +0000
@@ -24,8 +24,11 @@
 from cStringIO import StringIO
 
 
-import bzrlib
-import bzrlib.errors as errors
+from bzrlib import (
+    cache_utf8,
+    errors,
+    osutils,
+    )
 from bzrlib.store.revision import RevisionStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.versioned import VersionedFileStore
@@ -64,11 +67,16 @@
 
     def _add_revision(self, revision, revision_as_file, transaction):
         """Template method helper to store revision in this store."""
-        self.text_store.add(revision_as_file, revision.revision_id)
+        # TODO: jam 20070209 text_store.add() still requires a unicode file id
+        unicode_revision_id = cache_utf8.decode(revision.revision_id)
+        self.text_store.add(revision_as_file, unicode_revision_id)
 
     def add_revision_signature_text(self, revision_id, signature_text, transaction):
         """See RevisionStore.add_revision_signature_text()."""
-        self.text_store.add(StringIO(signature_text), revision_id, "sig")
+        # TODO: jam 20070209 text_store.add() still requires a unicode file id
+        revision_id_unicode = cache_utf8.decode(revision_id)
+        self.text_store.add(StringIO(signature_text), revision_id_unicode,
+                            "sig")
 
     def all_revision_ids(self, transaction):
         """See RevisionStore.all_revision_ids()."""
@@ -77,6 +85,7 @@
         assert self.text_store.listable()
         result_graph = {}
         for rev_id in self.text_store:
+            rev_id = osutils.safe_revision_id(rev_id)
             rev = self.get_revision(rev_id, transaction)
             result_graph[rev_id] = rev.parent_ids
         # remove ghosts
@@ -100,7 +109,7 @@
             xml_file.close()
             assert r.revision_id == revision_id
             revisions.append(r)
-        return revisions 
+        return revisions
 
     def _get_revision_xml_file(self, revision_id):
         try:
@@ -119,7 +128,7 @@
         """True if the store contains revision_id."""
         return (revision_id is None
                 or self.text_store.has_id(revision_id))
- 
+
     def _has_signature(self, revision_id, transaction):
         """See RevisionStore._has_signature()."""
         return self.text_store.has_id(revision_id, suffix='sig')

=== modified file 'bzrlib/tests/branch_implementations/test_branch.py'
--- bzrlib/tests/branch_implementations/test_branch.py	2007-02-12 23:01:49 +0000
+++ bzrlib/tests/branch_implementations/test_branch.py	2007-02-17 02:33:45 +0000
@@ -95,6 +95,27 @@
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
         self.assertRaises(errors.ReservedId, br.append_revision, 'current:')
 
+    def test_revision_ids_are_utf8(self):
+        wt = self.make_branch_and_tree('tree')
+        wt.commit('f', rev_id='rev1')
+        wt.commit('f', rev_id='rev2')
+        wt.commit('f', rev_id='rev3')
+
+        br = self.get_branch()
+        br.fetch(wt.branch)
+        br.set_revision_history(['rev1', 'rev2', 'rev3'])
+        rh = br.revision_history()
+        self.assertEqual(['rev1', 'rev2', 'rev3'], rh)
+        for revision_id in rh:
+            self.assertIsInstance(revision_id, str)
+        last = br.last_revision()
+        self.assertEqual('rev3', last)
+        self.assertIsInstance(last, str)
+        revno, last = br.last_revision_info()
+        self.assertEqual(3, revno)
+        self.assertEqual('rev3', last)
+        self.assertIsInstance(last, str)
+
     def test_fetch_revisions(self):
         """Test fetch-revision operation."""
         get_transport(self.get_url()).mkdir('b1')

=== modified file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	2007-02-13 05:22:39 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2007-02-17 02:02:41 +0000
@@ -47,6 +47,7 @@
         'bzrlib.tests.repository_implementations.test_break_lock',
         'bzrlib.tests.repository_implementations.test_commit_builder',
         'bzrlib.tests.repository_implementations.test_fileid_involved',
+        'bzrlib.tests.repository_implementations.test_iter_reverse_revision_history',
         'bzrlib.tests.repository_implementations.test_reconcile',
         'bzrlib.tests.repository_implementations.test_repository',
         'bzrlib.tests.repository_implementations.test_revision',

=== modified file 'bzrlib/tests/repository_implementations/test_commit_builder.py'
--- bzrlib/tests/repository_implementations/test_commit_builder.py	2006-11-28 01:56:31 +0000
+++ bzrlib/tests/repository_implementations/test_commit_builder.py	2007-02-10 02:48:43 +0000
@@ -56,7 +56,7 @@
         tree = self.make_branch_and_tree(".")
         # use a unicode revision id to test more corner cases.
         # The repository layer is meant to handle this.
-        revision_id = u'\xc8abc'
+        revision_id = u'\xc8abc'.encode('utf8')
         try:
             try:
                 builder = tree.branch.get_commit_builder([],

=== added file 'bzrlib/tests/repository_implementations/test_iter_reverse_revision_history.py'
--- bzrlib/tests/repository_implementations/test_iter_reverse_revision_history.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_iter_reverse_revision_history.py	2007-02-17 02:02:41 +0000
@@ -0,0 +1,145 @@
+# Copyright (C) 2007 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test iter_reverse_revision_history."""
+
+from bzrlib import (
+    errors,
+    tests,
+    )
+from bzrlib.tests.repository_implementations.test_repository import TestCaseWithRepository
+
+
+class TestIterReverseRevisionHistory(TestCaseWithRepository):
+
+    def create_linear_history(self):
+        tree = self.make_branch_and_memory_tree('tree')
+        tree.lock_write()
+        try:
+            tree.add('')
+            tree.commit('1', rev_id='rev1')
+            tree.commit('2', rev_id='rev2')
+            tree.commit('3', rev_id='rev3')
+            tree.commit('4', rev_id='rev4')
+        finally:
+            tree.unlock()
+        return tree
+
+    def create_linear_history_with_utf8(self):
+        tree = self.make_branch_and_memory_tree('tree')
+        tree.lock_write()
+        try:
+            tree.add('') # needed for MemoryTree
+            try:
+                tree.commit(u'F\xb5', rev_id=u'rev-\xb5'.encode('utf8'))
+            except errors.NonAsciiRevisionId:
+                raise tests.TestSkipped("%s doesn't support non-ascii"
+                                        " revision ids."
+                                        % self.repository_format)
+            tree.commit(u'B\xe5r', rev_id=u'rev-\xe5'.encode('utf8'))
+        finally:
+            tree.unlock()
+        return tree
+
+    def create_merged_history(self):
+        # TODO: jam 20070216 MutableTree doesn't yet have the pull() or
+        #       merge_from_branch() apis. So we have to use real trees for
+        #       this.
+        tree1 = self.make_branch_and_tree('tree1')
+        tree2 = self.make_branch_and_tree('tree2')
+        tree1.lock_write()
+        tree2.lock_write()
+        try:
+            tree1.add('')
+            tree1.commit('rev-1-1', rev_id='rev-1-1')
+            tree2.pull(tree1.branch)
+            tree2.commit('rev-2-2', rev_id='rev-2-2')
+            tree2.commit('rev-2-3', rev_id='rev-2-3')
+            tree2.commit('rev-2-4', rev_id='rev-2-4')
+
+            tree1.commit('rev-1-2', rev_id='rev-1-2')
+            tree1.merge_from_branch(tree2.branch)
+            tree1.commit('rev-1-3', rev_id='rev-1-3')
+
+            tree2.commit('rev-2-5', rev_id='rev-2-5')
+            # Make sure both repositories have all revisions
+            tree1.branch.repository.fetch(tree2.branch.repository,
+                                          revision_id='rev-2-5')
+            tree2.branch.repository.fetch(tree1.branch.repository,
+                                          revision_id='rev-1-3')
+        finally:
+            tree2.unlock()
+            tree1.unlock()
+        return tree1, tree2
+
+    def test_is_generator(self):
+        tree = self.create_linear_history()
+        repo = tree.branch.repository
+
+        rev_history = repo.iter_reverse_revision_history('rev4')
+        self.assertEqual('rev4', rev_history.next())
+        self.assertEqual('rev3', rev_history.next())
+        self.assertEqual('rev2', rev_history.next())
+        self.assertEqual('rev1', rev_history.next())
+        self.assertRaises(StopIteration, rev_history.next)
+
+    def assertRevHistoryList(self, expected, repo, revision_id):
+        """Assert the return values of iter_reverse_revision_history."""
+        actual = list(repo.iter_reverse_revision_history(revision_id))
+        self.assertEqual(expected, actual)
+
+    def test_linear_history(self):
+        tree = self.create_linear_history()
+        repo = tree.branch.repository
+
+        self.assertRevHistoryList(['rev4', 'rev3', 'rev2', 'rev1'],
+                                  repo, 'rev4')
+
+    def test_partial_history(self):
+        tree = self.create_linear_history()
+        repo = tree.branch.repository
+
+        self.assertRevHistoryList(['rev3', 'rev2', 'rev1'], repo, 'rev3')
+
+    def test_revision_ids_are_utf8(self):
+        tree = self.create_linear_history_with_utf8()
+        repo = tree.branch.repository
+
+        self.assertRevHistoryList(['rev-\xc3\xa5', 'rev-\xc2\xb5'],
+                                  repo, 'rev-\xc3\xa5')
+
+        # TODO: jam 20070216 Eventually unicode will be deprecated
+        self.assertRevHistoryList(['rev-\xc3\xa5', 'rev-\xc2\xb5'],
+                                  repo, u'rev-\xe5')
+
+    def test_merged_history(self):
+        tree1, tree2 = self.create_merged_history()
+        repo = tree1.branch.repository
+
+        self.assertRevHistoryList(['rev-1-1'],
+                                  repo, 'rev-1-1')
+        self.assertRevHistoryList(['rev-1-2', 'rev-1-1'],
+                                  repo, 'rev-1-2')
+        self.assertRevHistoryList(['rev-1-3', 'rev-1-2', 'rev-1-1'],
+                                  repo, 'rev-1-3')
+        self.assertRevHistoryList(['rev-2-2', 'rev-1-1'],
+                                  repo, 'rev-2-2')
+        self.assertRevHistoryList(['rev-2-3', 'rev-2-2', 'rev-1-1'],
+                                  repo, 'rev-2-3')
+        self.assertRevHistoryList(['rev-2-4', 'rev-2-3', 'rev-2-2', 'rev-1-1'],
+                                  repo, 'rev-2-4')
+        self.assertRevHistoryList(['rev-2-5', 'rev-2-4', 'rev-2-3', 'rev-2-2',
+                                   'rev-1-1'], repo, 'rev-2-5')

=== modified file 'bzrlib/tests/test_cache_utf8.py'
--- bzrlib/tests/test_cache_utf8.py	2006-08-10 00:57:21 +0000
+++ bzrlib/tests/test_cache_utf8.py	2007-02-09 16:41:00 +0000
@@ -78,6 +78,34 @@
         xp = cache_utf8.get_cached_unicode(x)
         yp = cache_utf8.get_cached_unicode(y)
 
-        self.failUnless(xp is x)
-        self.failUnless(xp is yp)
-
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
+
+    def test_cached_utf8(self):
+        x = u'\xb5yy\xe5zz'.encode('utf8')
+        y = u'\xb5yy\xe5zz'.encode('utf8')
+        self.failIf(x is y)
+        xp = cache_utf8.get_cached_utf8(x)
+        yp = cache_utf8.get_cached_utf8(y)
+
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
+
+    def test_cached_ascii(self):
+        x = '%s %s' % ('simple', 'text')
+        y = '%s %s' % ('simple', 'text')
+        self.failIf(x is y)
+        xp = cache_utf8.get_cached_ascii(x)
+        yp = cache_utf8.get_cached_ascii(y)
+
+        self.assertIs(xp, x)
+        self.assertIs(xp, yp)
+
+        # after caching, encode and decode should also return the right
+        # objects.
+        uni_x = cache_utf8.decode(x)
+        self.assertEqual(u'simple text', uni_x)
+        self.assertIsInstance(uni_x, unicode)
+
+        utf8_x = cache_utf8.encode(uni_x)
+        self.assertIs(utf8_x, x)

=== modified file 'bzrlib/tests/test_generate_ids.py'
--- bzrlib/tests/test_generate_ids.py	2006-11-02 22:48:49 +0000
+++ bzrlib/tests/test_generate_ids.py	2007-02-10 17:02:18 +0000
@@ -102,6 +102,10 @@
         """gen_revision_id should create a revision id matching the regex"""
         revision_id = generate_ids.gen_revision_id(username, timestamp)
         self.assertMatchesRe(regex, revision_id)
+        # It should be a utf8 revision_id, not a unicode one
+        self.assertIsInstance(revision_id, str)
+        # gen_revision_id should always return ascii revision ids.
+        revision_id.decode('ascii')
 
     def test_timestamp(self):
         """passing a timestamp should cause it to be used"""
@@ -123,8 +127,19 @@
     def test_gen_revision_id_user(self):
         """If there is no email, fall back to the whole username"""
         tail = r'-\d{14}-[a-z0-9]{16}'
-        self.assertGenRevisionId('joe_bar' + tail,'Joe Bar')
+        self.assertGenRevisionId('joe_bar' + tail, 'Joe Bar')
         self.assertGenRevisionId('joebar' + tail, 'joebar')
         self.assertGenRevisionId('joe_br' + tail, u'Joe B\xe5r')
         self.assertGenRevisionId(r'joe_br_user\+joe_bar_foo-bar.com' + tail,
                                  u'Joe B\xe5r <user+Joe_Bar_Foo-Bar.com>')
+
+    def test_revision_ids_are_ascii(self):
+        """gen_revision_id should always return an ascii revision id."""
+        tail = r'-\d{14}-[a-z0-9]{16}'
+        self.assertGenRevisionId('joe_bar' + tail, 'Joe Bar')
+        self.assertGenRevisionId('joe_bar' + tail, u'Joe Bar')
+        self.assertGenRevisionId('joe@foo' + tail, u'Joe Bar <joe@foo>')
+        # We cheat a little with this one, because email-addresses shouldn't
+        # contain non-ascii characters, but generate_ids should strip them
+        # anyway.
+        self.assertGenRevisionId('joe@f' + tail, u'Joe Bar <joe@f\xb6>')

=== modified file 'bzrlib/tests/test_knit.py'
--- bzrlib/tests/test_knit.py	2006-12-21 20:07:34 +0000
+++ bzrlib/tests/test_knit.py	2007-02-10 02:48:43 +0000
@@ -156,23 +156,27 @@
             transport.calls.pop(0))
 
     def test_read_utf8_version_id(self):
+        unicode_revision_id = u"version-\N{CYRILLIC CAPITAL LETTER A}"
+        utf8_revision_id = unicode_revision_id.encode('utf-8')
         transport = MockTransport([
             _KnitIndex.HEADER,
-            u"version-\N{CYRILLIC CAPITAL LETTER A}"
-                u" option 0 1 :".encode("utf-8")
+            '%s option 0 1 :' % (utf8_revision_id,)
             ])
         index = _KnitIndex(transport, "filename", "r")
-        self.assertTrue(
-            index.has_version(u"version-\N{CYRILLIC CAPITAL LETTER A}"))
+        # _KnitIndex is a private class, and deals in utf8 revision_ids, not
+        # Unicode revision_ids.
+        self.assertTrue(index.has_version(utf8_revision_id))
+        self.assertFalse(index.has_version(unicode_revision_id))
 
     def test_read_utf8_parents(self):
+        unicode_revision_id = u"version-\N{CYRILLIC CAPITAL LETTER A}"
+        utf8_revision_id = unicode_revision_id.encode('utf-8')
         transport = MockTransport([
             _KnitIndex.HEADER,
-            u"version option 0 1"
-                u" .version-\N{CYRILLIC CAPITAL LETTER A} :".encode("utf-8")
+            "version option 0 1 .%s :" % (utf8_revision_id,)
             ])
         index = _KnitIndex(transport, "filename", "r")
-        self.assertEqual([u"version-\N{CYRILLIC CAPITAL LETTER A}"],
+        self.assertEqual([utf8_revision_id],
             index.get_parents_with_ghosts("version"))
 
     def test_read_ignore_corrupted_lines(self):
@@ -184,7 +188,7 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
         self.assertEqual(1, index.num_versions())
-        self.assertTrue(index.has_version(u"version"))
+        self.assertTrue(index.has_version("version"))
 
     def test_read_corrupted_header(self):
         transport = MockTransport(['not a bzr knit index header\n'])
@@ -201,11 +205,11 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
         self.assertEqual(2, index.num_versions())
-        self.assertEqual(1, index.lookup(u"version"))
-        self.assertEqual((3, 4), index.get_position(u"version"))
-        self.assertEqual(["options3"], index.get_options(u"version"))
-        self.assertEqual([u"parent", u"other"],
-            index.get_parents_with_ghosts(u"version"))
+        self.assertEqual(1, index.lookup("version"))
+        self.assertEqual((3, 4), index.get_position("version"))
+        self.assertEqual(["options3"], index.get_options("version"))
+        self.assertEqual(["parent", "other"],
+            index.get_parents_with_ghosts("version"))
 
     def test_read_compressed_parents(self):
         transport = MockTransport([
@@ -215,32 +219,32 @@
             "c option 0 1 1 0 :",
             ])
         index = _KnitIndex(transport, "filename", "r")
-        self.assertEqual([u"a"], index.get_parents(u"b"))
-        self.assertEqual([u"b", u"a"], index.get_parents(u"c"))
+        self.assertEqual(["a"], index.get_parents("b"))
+        self.assertEqual(["b", "a"], index.get_parents("c"))
 
     def test_write_utf8_version_id(self):
+        unicode_revision_id = u"version-\N{CYRILLIC CAPITAL LETTER A}"
+        utf8_revision_id = unicode_revision_id.encode('utf-8')
         transport = MockTransport([
             _KnitIndex.HEADER
             ])
         index = _KnitIndex(transport, "filename", "r")
-        index.add_version(u"version-\N{CYRILLIC CAPITAL LETTER A}",
-            ["option"], 0, 1, [])
+        index.add_version(utf8_revision_id, ["option"], 0, 1, [])
         self.assertEqual(("append_bytes", ("filename",
-            u"\nversion-\N{CYRILLIC CAPITAL LETTER A}"
-                u" option 0 1  :".encode("utf-8")),
+            "\n%s option 0 1  :" % (utf8_revision_id,)),
             {}),
             transport.calls.pop(0))
 
     def test_write_utf8_parents(self):
+        unicode_revision_id = u"version-\N{CYRILLIC CAPITAL LETTER A}"
+        utf8_revision_id = unicode_revision_id.encode('utf-8')
         transport = MockTransport([
             _KnitIndex.HEADER
             ])
         index = _KnitIndex(transport, "filename", "r")
-        index.add_version(u"version", ["option"], 0, 1,
-            [u"version-\N{CYRILLIC CAPITAL LETTER A}"])
+        index.add_version("version", ["option"], 0, 1, [utf8_revision_id])
         self.assertEqual(("append_bytes", ("filename",
-            u"\nversion option 0 1"
-                u" .version-\N{CYRILLIC CAPITAL LETTER A} :".encode("utf-8")),
+            "\nversion option 0 1 .%s :" % (utf8_revision_id,)),
             {}),
             transport.calls.pop(0))
 
@@ -249,11 +253,11 @@
         index = _KnitIndex(transport, "filename", "w", create=True)
         self.assertEqual([], index.get_graph())
 
-        index.add_version(u"a", ["option"], 0, 1, [u"b"])
-        self.assertEqual([(u"a", [u"b"])], index.get_graph())
+        index.add_version("a", ["option"], 0, 1, ["b"])
+        self.assertEqual([("a", ["b"])], index.get_graph())
 
-        index.add_version(u"c", ["option"], 0, 1, [u"d"])
-        self.assertEqual([(u"a", [u"b"]), (u"c", [u"d"])],
+        index.add_version("c", ["option"], 0, 1, ["d"])
+        self.assertEqual([("a", ["b"]), ("c", ["d"])],
             sorted(index.get_graph()))
 
     def test_get_ancestry(self):
@@ -267,14 +271,14 @@
         index = _KnitIndex(transport, "filename", "r")
 
         self.assertEqual([], index.get_ancestry([]))
-        self.assertEqual([u"a"], index.get_ancestry([u"a"]))
-        self.assertEqual([u"a", u"b"], index.get_ancestry([u"b"]))
-        self.assertEqual([u"a", u"b", u"c"], index.get_ancestry([u"c"]))
-        self.assertEqual([u"a", u"b", u"c", u"d"], index.get_ancestry([u"d"]))
-        self.assertEqual([u"a", u"b"], index.get_ancestry([u"a", u"b"]))
-        self.assertEqual([u"a", u"b", u"c"], index.get_ancestry([u"a", u"c"]))
+        self.assertEqual(["a"], index.get_ancestry(["a"]))
+        self.assertEqual(["a", "b"], index.get_ancestry(["b"]))
+        self.assertEqual(["a", "b", "c"], index.get_ancestry(["c"]))
+        self.assertEqual(["a", "b", "c", "d"], index.get_ancestry(["d"]))
+        self.assertEqual(["a", "b"], index.get_ancestry(["a", "b"]))
+        self.assertEqual(["a", "b", "c"], index.get_ancestry(["a", "c"]))
 
-        self.assertRaises(RevisionNotPresent, index.get_ancestry, [u"e"])
+        self.assertRaises(RevisionNotPresent, index.get_ancestry, ["e"])
 
     def test_get_ancestry_with_ghosts(self):
         transport = MockTransport([
@@ -287,23 +291,23 @@
         index = _KnitIndex(transport, "filename", "r")
 
         self.assertEqual([], index.get_ancestry_with_ghosts([]))
-        self.assertEqual([u"a"], index.get_ancestry_with_ghosts([u"a"]))
-        self.assertEqual([u"a", u"e", u"b"],
-            index.get_ancestry_with_ghosts([u"b"]))
-        self.assertEqual([u"a", u"g", u"f", u"c"],
-            index.get_ancestry_with_ghosts([u"c"]))
-        self.assertEqual([u"a", u"g", u"f", u"c", u"k", u"j", u"h", u"d"],
-            index.get_ancestry_with_ghosts([u"d"]))
-        self.assertEqual([u"a", u"e", u"b"],
-            index.get_ancestry_with_ghosts([u"a", u"b"]))
-        self.assertEqual([u"a", u"g", u"f", u"c"],
-            index.get_ancestry_with_ghosts([u"a", u"c"]))
+        self.assertEqual(["a"], index.get_ancestry_with_ghosts(["a"]))
+        self.assertEqual(["a", "e", "b"],
+            index.get_ancestry_with_ghosts(["b"]))
+        self.assertEqual(["a", "g", "f", "c"],
+            index.get_ancestry_with_ghosts(["c"]))
+        self.assertEqual(["a", "g", "f", "c", "k", "j", "h", "d"],
+            index.get_ancestry_with_ghosts(["d"]))
+        self.assertEqual(["a", "e", "b"],
+            index.get_ancestry_with_ghosts(["a", "b"]))
+        self.assertEqual(["a", "g", "f", "c"],
+            index.get_ancestry_with_ghosts(["a", "c"]))
         self.assertEqual(
-            [u"a", u"g", u"f", u"c", u"e", u"b", u"k", u"j", u"h", u"d"],
-            index.get_ancestry_with_ghosts([u"b", u"d"]))
+            ["a", "g", "f", "c", "e", "b", "k", "j", "h", "d"],
+            index.get_ancestry_with_ghosts(["b", "d"]))
 
         self.assertRaises(RevisionNotPresent,
-            index.get_ancestry_with_ghosts, [u"e"])
+            index.get_ancestry_with_ghosts, ["e"])
 
     def test_num_versions(self):
         transport = MockTransport([
@@ -314,15 +318,15 @@
         self.assertEqual(0, index.num_versions())
         self.assertEqual(0, len(index))
 
-        index.add_version(u"a", ["option"], 0, 1, [])
-        self.assertEqual(1, index.num_versions())
-        self.assertEqual(1, len(index))
-
-        index.add_version(u"a", ["option2"], 1, 2, [])
-        self.assertEqual(1, index.num_versions())
-        self.assertEqual(1, len(index))
-
-        index.add_version(u"b", ["option"], 0, 1, [])
+        index.add_version("a", ["option"], 0, 1, [])
+        self.assertEqual(1, index.num_versions())
+        self.assertEqual(1, len(index))
+
+        index.add_version("a", ["option2"], 1, 2, [])
+        self.assertEqual(1, index.num_versions())
+        self.assertEqual(1, len(index))
+
+        index.add_version("b", ["option"], 0, 1, [])
         self.assertEqual(2, index.num_versions())
         self.assertEqual(2, len(index))
 
@@ -334,14 +338,14 @@
 
         self.assertEqual([], index.get_versions())
 
-        index.add_version(u"a", ["option"], 0, 1, [])
-        self.assertEqual([u"a"], index.get_versions())
-
-        index.add_version(u"a", ["option"], 0, 1, [])
-        self.assertEqual([u"a"], index.get_versions())
-
-        index.add_version(u"b", ["option"], 0, 1, [])
-        self.assertEqual([u"a", u"b"], index.get_versions())
+        index.add_version("a", ["option"], 0, 1, [])
+        self.assertEqual(["a"], index.get_versions())
+
+        index.add_version("a", ["option"], 0, 1, [])
+        self.assertEqual(["a"], index.get_versions())
+
+        index.add_version("b", ["option"], 0, 1, [])
+        self.assertEqual(["a", "b"], index.get_versions())
 
     def test_idx_to_name(self):
         transport = MockTransport([
@@ -351,10 +355,10 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual(u"a", index.idx_to_name(0))
-        self.assertEqual(u"b", index.idx_to_name(1))
-        self.assertEqual(u"b", index.idx_to_name(-1))
-        self.assertEqual(u"a", index.idx_to_name(-2))
+        self.assertEqual("a", index.idx_to_name(0))
+        self.assertEqual("b", index.idx_to_name(1))
+        self.assertEqual("b", index.idx_to_name(-1))
+        self.assertEqual("a", index.idx_to_name(-2))
 
     def test_lookup(self):
         transport = MockTransport([
@@ -364,8 +368,8 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual(0, index.lookup(u"a"))
-        self.assertEqual(1, index.lookup(u"b"))
+        self.assertEqual(0, index.lookup("a"))
+        self.assertEqual(1, index.lookup("b"))
 
     def test_add_version(self):
         transport = MockTransport([
@@ -373,35 +377,35 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        index.add_version(u"a", ["option"], 0, 1, [u"b"])
+        index.add_version("a", ["option"], 0, 1, ["b"])
         self.assertEqual(("append_bytes",
             ("filename", "\na option 0 1 .b :"),
             {}), transport.calls.pop(0))
-        self.assertTrue(index.has_version(u"a"))
+        self.assertTrue(index.has_version("a"))
         self.assertEqual(1, index.num_versions())
-        self.assertEqual((0, 1), index.get_position(u"a"))
-        self.assertEqual(["option"], index.get_options(u"a"))
-        self.assertEqual([u"b"], index.get_parents_with_ghosts(u"a"))
+        self.assertEqual((0, 1), index.get_position("a"))
+        self.assertEqual(["option"], index.get_options("a"))
+        self.assertEqual(["b"], index.get_parents_with_ghosts("a"))
 
-        index.add_version(u"a", ["opt"], 1, 2, [u"c"])
+        index.add_version("a", ["opt"], 1, 2, ["c"])
         self.assertEqual(("append_bytes",
             ("filename", "\na opt 1 2 .c :"),
             {}), transport.calls.pop(0))
-        self.assertTrue(index.has_version(u"a"))
+        self.assertTrue(index.has_version("a"))
         self.assertEqual(1, index.num_versions())
-        self.assertEqual((1, 2), index.get_position(u"a"))
-        self.assertEqual(["opt"], index.get_options(u"a"))
-        self.assertEqual([u"c"], index.get_parents_with_ghosts(u"a"))
+        self.assertEqual((1, 2), index.get_position("a"))
+        self.assertEqual(["opt"], index.get_options("a"))
+        self.assertEqual(["c"], index.get_parents_with_ghosts("a"))
 
-        index.add_version(u"b", ["option"], 2, 3, [u"a"])
+        index.add_version("b", ["option"], 2, 3, ["a"])
         self.assertEqual(("append_bytes",
             ("filename", "\nb option 2 3 0 :"),
             {}), transport.calls.pop(0))
-        self.assertTrue(index.has_version(u"b"))
+        self.assertTrue(index.has_version("b"))
         self.assertEqual(2, index.num_versions())
-        self.assertEqual((2, 3), index.get_position(u"b"))
-        self.assertEqual(["option"], index.get_options(u"b"))
-        self.assertEqual([u"a"], index.get_parents_with_ghosts(u"b"))
+        self.assertEqual((2, 3), index.get_position("b"))
+        self.assertEqual(["option"], index.get_options("b"))
+        self.assertEqual(["a"], index.get_parents_with_ghosts("b"))
 
     def test_add_versions(self):
         transport = MockTransport([
@@ -410,24 +414,24 @@
         index = _KnitIndex(transport, "filename", "r")
 
         index.add_versions([
-            (u"a", ["option"], 0, 1, [u"b"]),
-            (u"a", ["opt"], 1, 2, [u"c"]),
-            (u"b", ["option"], 2, 3, [u"a"])
+            ("a", ["option"], 0, 1, ["b"]),
+            ("a", ["opt"], 1, 2, ["c"]),
+            ("b", ["option"], 2, 3, ["a"])
             ])
         self.assertEqual(("append_bytes", ("filename",
             "\na option 0 1 .b :"
             "\na opt 1 2 .c :"
             "\nb option 2 3 0 :"
             ), {}), transport.calls.pop(0))
-        self.assertTrue(index.has_version(u"a"))
-        self.assertTrue(index.has_version(u"b"))
+        self.assertTrue(index.has_version("a"))
+        self.assertTrue(index.has_version("b"))
         self.assertEqual(2, index.num_versions())
-        self.assertEqual((1, 2), index.get_position(u"a"))
-        self.assertEqual((2, 3), index.get_position(u"b"))
-        self.assertEqual(["opt"], index.get_options(u"a"))
-        self.assertEqual(["option"], index.get_options(u"b"))
-        self.assertEqual([u"c"], index.get_parents_with_ghosts(u"a"))
-        self.assertEqual([u"a"], index.get_parents_with_ghosts(u"b"))
+        self.assertEqual((1, 2), index.get_position("a"))
+        self.assertEqual((2, 3), index.get_position("b"))
+        self.assertEqual(["opt"], index.get_options("a"))
+        self.assertEqual(["option"], index.get_options("b"))
+        self.assertEqual(["c"], index.get_parents_with_ghosts("a"))
+        self.assertEqual(["a"], index.get_parents_with_ghosts("b"))
 
     def test_delay_create_and_add_versions(self):
         transport = MockTransport()
@@ -438,9 +442,9 @@
         self.assertEqual([], transport.calls)
 
         index.add_versions([
-            (u"a", ["option"], 0, 1, [u"b"]),
-            (u"a", ["opt"], 1, 2, [u"c"]),
-            (u"b", ["option"], 2, 3, [u"a"])
+            ("a", ["option"], 0, 1, ["b"]),
+            ("a", ["opt"], 1, 2, ["c"]),
+            ("b", ["option"], 2, 3, ["a"])
             ])
         name, (filename, f), kwargs = transport.calls.pop(0)
         self.assertEqual("put_file_non_atomic", name)
@@ -462,8 +466,8 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertTrue(index.has_version(u"a"))
-        self.assertFalse(index.has_version(u"b"))
+        self.assertTrue(index.has_version("a"))
+        self.assertFalse(index.has_version("b"))
 
     def test_get_position(self):
         transport = MockTransport([
@@ -473,8 +477,8 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual((0, 1), index.get_position(u"a"))
-        self.assertEqual((1, 2), index.get_position(u"b"))
+        self.assertEqual((0, 1), index.get_position("a"))
+        self.assertEqual((1, 2), index.get_position("b"))
 
     def test_get_method(self):
         transport = MockTransport([
@@ -485,9 +489,9 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual("fulltext", index.get_method(u"a"))
-        self.assertEqual("line-delta", index.get_method(u"b"))
-        self.assertRaises(errors.KnitIndexUnknownMethod, index.get_method, u"c")
+        self.assertEqual("fulltext", index.get_method("a"))
+        self.assertEqual("line-delta", index.get_method("b"))
+        self.assertRaises(errors.KnitIndexUnknownMethod, index.get_method, "c")
 
     def test_get_options(self):
         transport = MockTransport([
@@ -497,8 +501,8 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual(["opt1"], index.get_options(u"a"))
-        self.assertEqual(["opt2", "opt3"], index.get_options(u"b"))
+        self.assertEqual(["opt1"], index.get_options("a"))
+        self.assertEqual(["opt2", "opt3"], index.get_options("b"))
 
     def test_get_parents(self):
         transport = MockTransport([
@@ -509,9 +513,9 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual([], index.get_parents(u"a"))
-        self.assertEqual([u"a", u"c"], index.get_parents(u"b"))
-        self.assertEqual([u"b", u"a"], index.get_parents(u"c"))
+        self.assertEqual([], index.get_parents("a"))
+        self.assertEqual(["a", "c"], index.get_parents("b"))
+        self.assertEqual(["b", "a"], index.get_parents("c"))
 
     def test_get_parents_with_ghosts(self):
         transport = MockTransport([
@@ -522,10 +526,10 @@
             ])
         index = _KnitIndex(transport, "filename", "r")
 
-        self.assertEqual([], index.get_parents_with_ghosts(u"a"))
-        self.assertEqual([u"a", u"c"], index.get_parents_with_ghosts(u"b"))
-        self.assertEqual([u"b", u"a", u"e"],
-            index.get_parents_with_ghosts(u"c"))
+        self.assertEqual([], index.get_parents_with_ghosts("a"))
+        self.assertEqual(["a", "c"], index.get_parents_with_ghosts("b"))
+        self.assertEqual(["b", "a", "e"],
+            index.get_parents_with_ghosts("c"))
 
     def test_check_versions_present(self):
         transport = MockTransport([
@@ -538,11 +542,11 @@
         check = index.check_versions_present
 
         check([])
-        check([u"a"])
-        check([u"b"])
-        check([u"a", u"b"])
-        self.assertRaises(RevisionNotPresent, check, [u"c"])
-        self.assertRaises(RevisionNotPresent, check, [u"a", u"b", u"c"])
+        check(["a"])
+        check(["b"])
+        check(["a", "b"])
+        self.assertRaises(RevisionNotPresent, check, ["c"])
+        self.assertRaises(RevisionNotPresent, check, ["a", "b", "c"])
 
 
 class KnitTests(TestCaseWithTransport):

=== modified file 'bzrlib/tests/test_lockable_files.py'
--- bzrlib/tests/test_lockable_files.py	2006-10-11 23:08:27 +0000
+++ bzrlib/tests/test_lockable_files.py	2007-02-09 23:39:24 +0000
@@ -56,10 +56,13 @@
                               StringIO(unicode_string)
                               )
             self.lockable.put_utf8('bar', unicode_string)
-            self.assertEqual(unicode_string, 
+            self.assertEqual(unicode_string,
                              self.lockable.get_utf8('bar').read())
-            self.assertEqual(byte_string, 
+            self.assertEqual(byte_string,
                              self.lockable.get('bar').read())
+            self.lockable.put_bytes('raw', 'raw\xffbytes')
+            self.assertEqual('raw\xffbytes',
+                             self.lockable.get('raw').read())
         finally:
             self.lockable.unlock()
 

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2007-02-14 10:09:00 +0000
+++ bzrlib/tests/test_osutils.py	2007-02-17 01:42:57 +0000
@@ -261,6 +261,54 @@
                           '\xbb\xbb')
 
 
+class TestSafeUtf8(TestCase):
+
+    def test_from_ascii_string(self):
+        f = 'foobar'
+        self.assertEqual('foobar', osutils.safe_utf8(f))
+
+    def test_from_unicode_string_ascii_contents(self):
+        self.assertEqual('bargam', osutils.safe_utf8(u'bargam'))
+
+    def test_from_unicode_string_unicode_contents(self):
+        self.assertEqual('bargam\xc2\xae', osutils.safe_utf8(u'bargam\xae'))
+
+    def test_from_utf8_string(self):
+        self.assertEqual('foo\xc2\xae', osutils.safe_utf8('foo\xc2\xae'))
+
+    def test_bad_utf8_string(self):
+        self.assertRaises(BzrBadParameterNotUnicode,
+                          osutils.safe_utf8, '\xbb\xbb')
+
+
+class TestSafeRevisionId(TestCase):
+
+    def test_from_ascii_string(self):
+        f = 'foobar'
+        self.assertEqual('foobar', osutils.safe_revision_id(f))
+        self.assertIs(osutils.safe_utf8(f), f)
+
+    def test_from_unicode_string_ascii_contents(self):
+        self.assertEqual('bargam', osutils.safe_revision_id(u'bargam'))
+
+    def test_from_unicode_string_unicode_contents(self):
+        self.assertEqual('bargam\xc2\xae',
+                         osutils.safe_revision_id(u'bargam\xae'))
+
+    def test_from_utf8_string(self):
+        self.assertEqual('foo\xc2\xae',
+                         osutils.safe_revision_id('foo\xc2\xae'))
+
+    def test_bad_utf8_string(self):
+        # This check may eventually go away
+        self.assertRaises(BzrBadParameterNotUnicode,
+                          osutils.safe_utf8, '\xbb\xbb')
+
+    def test_none(self):
+        """Currently, None is a valid revision_id"""
+        self.assertEqual(None, osutils.safe_revision_id(None))
+
+
 class TestWin32Funcs(TestCase):
     """Test that the _win32 versions of os utilities return appropriate paths."""
 

=== modified file 'bzrlib/tests/test_versionedfile.py'
--- bzrlib/tests/test_versionedfile.py	2007-01-11 17:20:25 +0000
+++ bzrlib/tests/test_versionedfile.py	2007-02-13 20:33:57 +0000
@@ -213,36 +213,36 @@
         self.assertEqual(expected_delta, deltas['noeol'])
         # smoke tests for eol support - two noeol in a row same content
         expected_deltas = (('noeol', '3ad7ee82dbd8f29ecba073f96e43e414b3f70a4d', True, 
-                          [(0, 1, 2, [(u'noeolsecond', 'line\n'), (u'noeolsecond', 'line\n')])]),
+                          [(0, 1, 2, [('noeolsecond', 'line\n'), ('noeolsecond', 'line\n')])]),
                           ('noeol', '3ad7ee82dbd8f29ecba073f96e43e414b3f70a4d', True, 
                            [(0, 0, 1, [('noeolsecond', 'line\n')]), (1, 1, 0, [])]))
         self.assertEqual(['line\n', 'line'], f.get_lines('noeolsecond'))
         self.assertTrue(deltas['noeolsecond'] in expected_deltas)
         # two no-eol in a row, different content
         expected_delta = ('noeolsecond', '8bb553a84e019ef1149db082d65f3133b195223b', True, 
-                          [(1, 2, 1, [(u'noeolnotshared', 'phone\n')])])
+                          [(1, 2, 1, [('noeolnotshared', 'phone\n')])])
         self.assertEqual(['line\n', 'phone'], f.get_lines('noeolnotshared'))
         self.assertEqual(expected_delta, deltas['noeolnotshared'])
         # eol folling a no-eol with content change
         expected_delta = ('noeol', 'a61f6fb6cfc4596e8d88c34a308d1e724caf8977', False, 
-                          [(0, 1, 1, [(u'eol', 'phone\n')])])
+                          [(0, 1, 1, [('eol', 'phone\n')])])
         self.assertEqual(['phone\n'], f.get_lines('eol'))
         self.assertEqual(expected_delta, deltas['eol'])
         # eol folling a no-eol with content change
         expected_delta = ('noeol', '6bfa09d82ce3e898ad4641ae13dd4fdb9cf0d76b', False, 
-                          [(0, 1, 1, [(u'eolline', 'line\n')])])
+                          [(0, 1, 1, [('eolline', 'line\n')])])
         self.assertEqual(['line\n'], f.get_lines('eolline'))
         self.assertEqual(expected_delta, deltas['eolline'])
         # eol with no parents
         expected_delta = (None, '264f39cab871e4cfd65b3a002f7255888bb5ed97', True, 
-                          [(0, 0, 1, [(u'noeolbase', 'line\n')])])
+                          [(0, 0, 1, [('noeolbase', 'line\n')])])
         self.assertEqual(['line'], f.get_lines('noeolbase'))
         self.assertEqual(expected_delta, deltas['noeolbase'])
         # eol with two parents, in inverse insertion order
         expected_deltas = (('noeolbase', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True,
-                            [(0, 1, 1, [(u'eolbeforefirstparent', 'line\n')])]),
+                            [(0, 1, 1, [('eolbeforefirstparent', 'line\n')])]),
                            ('noeolbase', '264f39cab871e4cfd65b3a002f7255888bb5ed97', True,
-                            [(0, 1, 1, [(u'eolbeforefirstparent', 'line\n')])]))
+                            [(0, 1, 1, [('eolbeforefirstparent', 'line\n')])]))
         self.assertEqual(['line'], f.get_lines('eolbeforefirstparent'))
         #self.assertTrue(deltas['eolbeforefirstparent'] in expected_deltas)
 
@@ -668,8 +668,11 @@
         # add_lines_with_ghosts api.
         vf = self.get_file()
         # add a revision with ghost parents
+        # The preferred form is utf8, but we should translate when needed
+        parent_id_unicode = u'b\xbfse'
+        parent_id_utf8 = parent_id_unicode.encode('utf8')
         try:
-            vf.add_lines_with_ghosts(u'notbxbfse', [u'b\xbfse'], [])
+            vf.add_lines_with_ghosts(u'notbxbfse', [parent_id_utf8], [])
         except NotImplementedError:
             # check the other ghost apis are also not implemented
             self.assertRaises(NotImplementedError, vf.has_ghost, 'foo')
@@ -681,33 +684,37 @@
         # test key graph related apis: getncestry, _graph, get_parents
         # has_version
         # - these are ghost unaware and must not be reflect ghosts
-        self.assertEqual([u'notbxbfse'], vf.get_ancestry(u'notbxbfse'))
-        self.assertEqual([], vf.get_parents(u'notbxbfse'))
-        self.assertEqual({u'notbxbfse':[]}, vf.get_graph())
-        self.assertFalse(vf.has_version(u'b\xbfse'))
+        self.assertEqual(['notbxbfse'], vf.get_ancestry('notbxbfse'))
+        self.assertEqual([], vf.get_parents('notbxbfse'))
+        self.assertEqual({'notbxbfse':[]}, vf.get_graph())
+        self.assertFalse(vf.has_version(parent_id_unicode))
+        self.assertFalse(vf.has_version(parent_id_utf8))
         # we have _with_ghost apis to give us ghost information.
-        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry_with_ghosts([u'notbxbfse']))
-        self.assertEqual([u'b\xbfse'], vf.get_parents_with_ghosts(u'notbxbfse'))
-        self.assertEqual({u'notbxbfse':[u'b\xbfse']}, vf.get_graph_with_ghosts())
-        self.assertTrue(vf.has_ghost(u'b\xbfse'))
+        self.assertEqual([parent_id_utf8, 'notbxbfse'], vf.get_ancestry_with_ghosts(['notbxbfse']))
+        self.assertEqual([parent_id_utf8], vf.get_parents_with_ghosts('notbxbfse'))
+        self.assertEqual({'notbxbfse':[parent_id_utf8]}, vf.get_graph_with_ghosts())
+        self.assertTrue(vf.has_ghost(parent_id_unicode))
+        self.assertTrue(vf.has_ghost(parent_id_utf8))
         # if we add something that is a ghost of another, it should correct the
         # results of the prior apis
-        vf.add_lines(u'b\xbfse', [], [])
-        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry([u'notbxbfse']))
-        self.assertEqual([u'b\xbfse'], vf.get_parents(u'notbxbfse'))
-        self.assertEqual({u'b\xbfse':[],
-                          u'notbxbfse':[u'b\xbfse'],
+        vf.add_lines(parent_id_unicode, [], [])
+        self.assertEqual([parent_id_utf8, 'notbxbfse'], vf.get_ancestry(['notbxbfse']))
+        self.assertEqual([parent_id_utf8], vf.get_parents('notbxbfse'))
+        self.assertEqual({parent_id_utf8:[],
+                          'notbxbfse':[parent_id_utf8],
                           },
                          vf.get_graph())
-        self.assertTrue(vf.has_version(u'b\xbfse'))
+        self.assertTrue(vf.has_version(parent_id_unicode))
+        self.assertTrue(vf.has_version(parent_id_utf8))
         # we have _with_ghost apis to give us ghost information.
-        self.assertEqual([u'b\xbfse', u'notbxbfse'], vf.get_ancestry_with_ghosts([u'notbxbfse']))
-        self.assertEqual([u'b\xbfse'], vf.get_parents_with_ghosts(u'notbxbfse'))
-        self.assertEqual({u'b\xbfse':[],
-                          u'notbxbfse':[u'b\xbfse'],
+        self.assertEqual([parent_id_utf8, 'notbxbfse'], vf.get_ancestry_with_ghosts(['notbxbfse']))
+        self.assertEqual([parent_id_utf8], vf.get_parents_with_ghosts('notbxbfse'))
+        self.assertEqual({parent_id_utf8:[],
+                          'notbxbfse':[parent_id_utf8],
                           },
                          vf.get_graph_with_ghosts())
-        self.assertFalse(vf.has_ghost(u'b\xbfse'))
+        self.assertFalse(vf.has_ghost(parent_id_unicode))
+        self.assertFalse(vf.has_ghost(parent_id_utf8))
 
     def test_add_lines_with_ghosts_after_normal_revs(self):
         # some versioned file formats allow lines to be added with parent

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/tests/test_xml.py	2006-11-12 15:01:27 +0000
+++ bzrlib/tests/test_xml.py	2007-02-09 23:15:53 +0000
@@ -259,3 +259,53 @@
         txt = s_v5.write_revision_to_string(rev)
         new_rev = s_v5.read_revision_from_string(txt)
         self.assertEqual(props, new_rev.properties)
+
+    def test_revision_ids_are_utf8(self):
+        """Parsed revision_ids should all be utf-8 strings, not unicode."""
+        s_v5 = bzrlib.xml5.serializer_v5
+        rev = s_v5.read_revision_from_string(_revision_v5)
+        self.assertIsInstance(rev.revision_id, str)
+        for parent_id in rev.parent_ids:
+            self.assertIsInstance(parent_id, str)
+
+        # ie.revision should either be None or a utf-8 revision id
+        inv = s_v5.read_inventory_from_string(_committed_inv_v5)
+        for path, ie in inv.iter_entries():
+            if ie.revision is None:
+                continue
+            self.assertIsInstance(ie.revision, str)
+
+
+class TestEncodeAndEscape(TestCase):
+    """Whitebox testing of the _encode_and_escape function."""
+
+    def setUp(self):
+        # Keep the cache clear before and after the test
+        bzrlib.xml5._ensure_utf8_re()
+        bzrlib.xml5._clear_cache()
+        self.addCleanup(bzrlib.xml5._clear_cache)
+
+    def test_simple_ascii(self):
+        # _encode_and_escape always appends a final ", because these parameters
+        # are being used in xml attributes, and by returning it now, we have to
+        # do fewer string operations later.
+        val = bzrlib.xml5._encode_and_escape('foo bar')
+        self.assertEqual('foo bar"', val)
+        # The second time should be cached
+        val2 = bzrlib.xml5._encode_and_escape('foo bar')
+        self.assertIs(val2, val)
+
+    def test_ascii_with_xml(self):
+        self.assertEqual('&amp;&apos;&quot;&lt;&gt;"',
+                         bzrlib.xml5._encode_and_escape('&\'"<>'))
+
+    def test_utf8_with_xml(self):
+        # u'\xb5\xe5&\u062c'
+        utf8_str = '\xc2\xb5\xc3\xa5&\xd8\xac'
+        self.assertEqual('&#181;&#229;&amp;&#1580;"',
+                         bzrlib.xml5._encode_and_escape(utf8_str))
+
+    def test_unicode(self):
+        uni_str = u'\xb5\xe5&\u062c'
+        self.assertEqual('&#181;&#229;&amp;&#1580;"',
+                         bzrlib.xml5._encode_and_escape(uni_str))

=== modified file 'bzrlib/tests/workingtree_implementations/test_get_parent_ids.py'
--- bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	2006-10-05 05:37:25 +0000
+++ bzrlib/tests/workingtree_implementations/test_get_parent_ids.py	2007-02-09 17:24:23 +0000
@@ -32,6 +32,8 @@
         self.assertEqual([rev2_id], t2.get_parent_ids())
         t.merge_from_branch(t2.branch)
         self.assertEqual([rev1_id, rev2_id], t.get_parent_ids())
+        for parent_id in t.get_parent_ids():
+            self.assertIsInstance(parent_id, str)
 
     def test_pending_merges(self):
         """Test the correspondence between set pending merges and get_parent_ids."""

=== modified file 'bzrlib/tests/workingtree_implementations/test_workingtree.py'
--- bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-02-09 20:49:11 +0000
+++ bzrlib/tests/workingtree_implementations/test_workingtree.py	2007-02-17 01:42:57 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007 Canonical Ltd
 # Authors:  Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -269,7 +269,10 @@
         wt = self.make_branch_and_tree('source')
         self.assertEqual([], wt.get_parent_ids())
         wt.commit('A', allow_pointless=True, rev_id='A')
-        self.assertEqual(['A'], wt.get_parent_ids())
+        parent_ids = wt.get_parent_ids()
+        self.assertEqual(['A'], parent_ids)
+        for parent_id in parent_ids:
+            self.assertIsInstance(parent_id, str)
 
     def test_set_last_revision(self):
         wt = self.make_branch_and_tree('source')

=== modified file 'bzrlib/versionedfile.py'
--- bzrlib/versionedfile.py	2007-01-17 15:37:08 +0000
+++ bzrlib/versionedfile.py	2007-02-10 02:48:43 +0000
@@ -26,6 +26,7 @@
 
 from bzrlib import (
     errors,
+    osutils,
     tsort,
     revision,
     ui,
@@ -96,6 +97,8 @@
         :param sha1: The sha1 of the full text.
         :param delta: The delta instructions. See get_delta for details.
         """
+        version_id = osutils.safe_revision_id(version_id)
+        parents = [osutils.safe_revision_id(v) for v in parents]
         self._check_write_ok()
         if self.has_version(version_id):
             raise errors.RevisionAlreadyPresent(version_id, self)
@@ -138,6 +141,8 @@
                  provided back to future add_lines calls in the parent_texts
                  dictionary.
         """
+        version_id = osutils.safe_revision_id(version_id)
+        parents = [osutils.safe_revision_id(v) for v in parents]
         self._check_write_ok()
         return self._add_lines(version_id, parents, lines, parent_texts)
 
@@ -151,6 +156,8 @@
         
         This takes the same parameters as add_lines.
         """
+        version_id = osutils.safe_revision_id(version_id)
+        parents = [osutils.safe_revision_id(v) for v in parents]
         self._check_write_ok()
         return self._add_lines_with_ghosts(version_id, parents, lines,
                                            parent_texts)
@@ -204,6 +211,8 @@
 
         Must raise RevisionAlreadyPresent if the new version is
         already present in file history."""
+        new_version_id = osutils.safe_revision_id(new_version_id)
+        old_version_id = osutils.safe_revision_id(old_version_id)
         self._check_write_ok()
         return self._clone_text(new_version_id, old_version_id, parents)
 
@@ -220,7 +229,7 @@
         """
         raise NotImplementedError(self.create_empty)
 
-    def fix_parents(self, version, new_parents):
+    def fix_parents(self, version_id, new_parents):
         """Fix the parents list for version.
         
         This is done by appending a new version to the index
@@ -228,10 +237,12 @@
         the parents list must be a superset of the current
         list.
         """
+        version_id = osutils.safe_revision_id(version_id)
+        new_parents = [osutils.safe_revision_id(p) for p in new_parents]
         self._check_write_ok()
-        return self._fix_parents(version, new_parents)
+        return self._fix_parents(version_id, new_parents)
 
-    def _fix_parents(self, version, new_parents):
+    def _fix_parents(self, version_id, new_parents):
         """Helper for fix_parents."""
         raise NotImplementedError(self.fix_parents)
 
@@ -243,7 +254,7 @@
         """
         raise NotImplementedError(self.get_delta)
 
-    def get_deltas(self, versions):
+    def get_deltas(self, version_ids):
         """Get multiple deltas at once for constructing versions.
         
         :return: dict(version_id:(delta_parent, sha1, noeol, delta))
@@ -251,8 +262,8 @@
         version_id is the version_id created by that delta.
         """
         result = {}
-        for version in versions:
-            result[version] = self.get_delta(version)
+        for version_id in version_ids:
+            result[version_id] = self.get_delta(version_id)
         return result
 
     def get_sha1(self, version_id):
@@ -325,7 +336,7 @@
             for version in self.versions():
                 result[version] = self.get_parents(version)
         else:
-            pending = set(version_ids)
+            pending = set(osutils.safe_revision_id(v) for v in version_ids)
             while pending:
                 version = pending.pop()
                 if version in result:
@@ -657,6 +668,7 @@
             # None cannot be in source.versions
             return set(self.source.versions())
         else:
+            version_ids = [osutils.safe_revision_id(v) for v in version_ids]
             if ignore_missing:
                 return set(self.source.versions()).intersection(set(version_ids))
             else:

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2007-02-09 16:26:22 +0000
+++ bzrlib/workingtree.py	2007-02-17 01:42:57 +0000
@@ -503,12 +503,13 @@
         else:
             parents = [last_rev]
         try:
-            merges_file = self._control_files.get_utf8('pending-merges')
+            merges_file = self._control_files.get('pending-merges')
         except errors.NoSuchFile:
             pass
         else:
             for l in merges_file.readlines():
-                parents.append(l.rstrip('\n'))
+                revision_id = osutils.safe_revision_id(l.rstrip('\n'))
+                parents.append(revision_id)
         return parents
 
     @needs_read_lock
@@ -719,6 +720,7 @@
         :param revision_ids: The revision_ids to set as the parent ids of this
             working tree. Any of these may be ghosts.
         """
+        revision_ids = [osutils.safe_revision_id(r) for r in revision_ids]
         self._check_parents_for_ghosts(revision_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
 
@@ -732,7 +734,7 @@
     @needs_tree_write_lock
     def set_parent_trees(self, parents_list, allow_leftmost_as_ghost=False):
         """See MutableTree.set_parent_trees."""
-        parent_ids = [rev for (rev, tree) in parents_list]
+        parent_ids = [osutils.safe_revision_id(rev) for (rev, tree) in parents_list]
 
         self._check_parents_for_ghosts(parent_ids,
             allow_leftmost_as_ghost=allow_leftmost_as_ghost)
@@ -795,6 +797,8 @@
             merger.check_basis(check_clean=True, require_commits=False)
             if to_revision is None:
                 to_revision = branch.last_revision()
+            else:
+                to_revision = osutils.safe_revision_id(to_revision)
             merger.other_rev_id = to_revision
             if merger.other_rev_id is None:
                 raise error.NoCommits(branch)
@@ -1532,6 +1536,7 @@
     @needs_tree_write_lock
     def set_last_revision(self, new_revision):
         """Change the last revision in the working tree."""
+        new_revision = osutils.safe_revision_id(new_revision)
         if self._change_last_revision(new_revision):
             self._cache_basis_inventory(new_revision)
 
@@ -1560,7 +1565,10 @@
 
     def _create_basis_xml_from_inventory(self, revision_id, inventory):
         """Create the text that will be saved in basis-inventory"""
-        inventory.revision_id = revision_id
+        # TODO: jam 20070209 This should be redundant, as the revision_id
+        #       as all callers should have already converted the revision_id to
+        #       utf8
+        inventory.revision_id = osutils.safe_revision_id(revision_id)
         return xml6.serializer_v6.write_inventory_to_string(inventory)
 
     def _cache_basis_inventory(self, new_revision):
@@ -1953,7 +1961,8 @@
     def _last_revision(self):
         """See Mutable.last_revision."""
         try:
-            return self._control_files.get_utf8('last-revision').read()
+            return osutils.safe_revision_id(
+                        self._control_files.get('last-revision').read())
         except errors.NoSuchFile:
             return None
 
@@ -2133,6 +2142,7 @@
             raise errors.NotLocalUrl(a_bzrdir.transport.base)
         branch = a_bzrdir.open_branch()
         if revision_id is not None:
+            revision_id = osutils.safe_revision_id(revision_id)
             branch.lock_write()
             try:
                 revision_history = branch.revision_history()
@@ -2224,6 +2234,8 @@
         branch = a_bzrdir.open_branch()
         if revision_id is None:
             revision_id = branch.last_revision()
+        else:
+            revision_id = osutils.safe_revision_id(revision_id)
         # WorkingTree3 can handle an inventory which has a unique root id.
         # as of bzr 0.12. However, bzr 0.11 and earlier fail to handle
         # those trees. And because there isn't a format bump inbetween, we

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2006-11-12 15:01:27 +0000
+++ bzrlib/xml5.py	2007-02-09 23:15:53 +0000
@@ -28,7 +28,8 @@
 
 
 _utf8_re = None
-_utf8_escape_map = {
+_unicode_re = None
+_xml_escape_map = {
     "&":'&amp;',
     "'":"&apos;", # FIXME: overkill
     "\"":"&quot;",
@@ -38,14 +39,15 @@
 
 
 def _ensure_utf8_re():
-    """Make sure the _utf8_re regex has been compiled"""
-    global _utf8_re
-    if _utf8_re is not None:
-        return
-    _utf8_re = re.compile(u'[&<>\'\"\u0080-\uffff]')
-
-
-def _utf8_escape_replace(match, _map=_utf8_escape_map):
+    """Make sure the _utf8_re and _unicode_re regexes have been compiled."""
+    global _utf8_re, _unicode_re
+    if _utf8_re is None:
+        _utf8_re = re.compile('[&<>\'\"]|[\x80-\xff]+')
+    if _unicode_re is None:
+        _unicode_re = re.compile(u'[&<>\'\"\u0080-\uffff]')
+
+
+def _unicode_escape_replace(match, _map=_xml_escape_map):
     """Replace a string of non-ascii, non XML safe characters with their escape
 
     This will escape both Standard XML escapes, like <>"', etc.
@@ -64,29 +66,74 @@
         return "&#%d;" % ord(match.group())
 
 
-_unicode_to_escaped_map = {}
-
-def _encode_and_escape(unicode_str, _map=_unicode_to_escaped_map):
+def _utf8_escape_replace(match, _map=_xml_escape_map):
+    """Escape utf8 characters into XML safe ones.
+
+    This uses 2 tricks. It is either escaping "standard" characters, like "&<>,
+    or it is handling characters with the high-bit set. For ascii characters,
+    we just lookup the replacement in the dictionary. For everything else, we
+    decode back into Unicode, and then use the XML escape code.
+    """
+    try:
+        return _map[match.group()]
+    except KeyError:
+        return ''.join('&#%d;' % ord(uni_chr)
+                       for uni_chr in match.group().decode('utf8'))
+
+
+_to_escaped_map = {}
+
+def _encode_and_escape(unicode_or_utf8_str, _map=_to_escaped_map):
     """Encode the string into utf8, and escape invalid XML characters"""
     # We frequently get entities we have not seen before, so it is better
     # to check if None, rather than try/KeyError
-    text = _map.get(unicode_str)
+    text = _map.get(unicode_or_utf8_str)
     if text is None:
-        # The alternative policy is to do a regular UTF8 encoding
-        # and then escape only XML meta characters.
-        # Performance is equivalent once you use cache_utf8. *However*
-        # this makes the serialized texts incompatible with old versions
-        # of bzr. So no net gain. (Perhaps the read code would handle utf8
-        # better than entity escapes, but cElementTree seems to do just fine
-        # either way)
-        text = str(_utf8_re.sub(_utf8_escape_replace, unicode_str)) + '"'
-        _map[unicode_str] = text
+        if unicode_or_utf8_str.__class__ == unicode:
+            # The alternative policy is to do a regular UTF8 encoding
+            # and then escape only XML meta characters.
+            # Performance is equivalent once you use cache_utf8. *However*
+            # this makes the serialized texts incompatible with old versions
+            # of bzr. So no net gain. (Perhaps the read code would handle utf8
+            # better than entity escapes, but cElementTree seems to do just fine
+            # either way)
+            text = str(_unicode_re.sub(_unicode_escape_replace,
+                                       unicode_or_utf8_str)) + '"'
+        else:
+            # Plain strings are considered to already be in utf-8 so we do a
+            # slightly different method for escaping.
+            text = _utf8_re.sub(_utf8_escape_replace,
+                                unicode_or_utf8_str) + '"'
+        _map[unicode_or_utf8_str] = text
     return text
 
 
+def _get_utf8_or_ascii(a_str,
+                       _encode_utf8=cache_utf8.encode,
+                       _get_cached_ascii=cache_utf8.get_cached_ascii):
+    """Return a cached version of the string.
+
+    cElementTree will return a plain string if the XML is plain ascii. It only
+    returns Unicode when it needs to. We want to work in utf-8 strings. So if
+    cElementTree returns a plain string, we can just return the cached version.
+    If it is Unicode, then we need to encode it.
+
+    :param a_str: An 8-bit string or Unicode as returned by
+                  cElementTree.Element.get()
+    :return: A utf-8 encoded 8-bit string.
+    """
+    # This is fairly optimized because we know what cElementTree does, this is
+    # not meant as a generic function for all cases. Because it is possible for
+    # an 8-bit string to not be ascii or valid utf8.
+    if a_str.__class__ == unicode:
+        return _encode_utf8(a_str)
+    else:
+        return _get_cached_ascii(a_str)
+
+
 def _clear_cache():
     """Clean out the unicode => escaped map"""
-    _unicode_to_escaped_map.clear()
+    _to_escaped_map.clear()
 
 
 class Serializer_v5(Serializer):
@@ -178,10 +225,17 @@
 
     def _pack_revision(self, rev):
         """Revision object -> xml tree"""
+        # For the XML format, we need to write them as Unicode rather than as
+        # utf-8 strings. So that cElementTree can handle properly escaping
+        # them.
+        decode_utf8 = cache_utf8.decode
+        revision_id = rev.revision_id
+        if isinstance(revision_id, str):
+            revision_id = decode_utf8(revision_id)
         root = Element('revision',
                        committer = rev.committer,
                        timestamp = '%.3f' % rev.timestamp,
-                       revision_id = rev.revision_id,
+                       revision_id = revision_id,
                        inventory_sha1 = rev.inventory_sha1,
                        format='5',
                        )
@@ -198,6 +252,8 @@
                 assert isinstance(parent_id, basestring)
                 p = SubElement(pelts, 'revision_ref')
                 p.tail = '\n'
+                if isinstance(parent_id, str):
+                    parent_id = decode_utf8(parent_id)
                 p.set('revision_id', parent_id)
         if rev.properties:
             self._pack_revision_properties(rev, root)
@@ -226,7 +282,7 @@
                                 % format)
         revision_id = elt.get('revision_id')
         if revision_id is not None:
-            revision_id = cache_utf8.get_cached_unicode(revision_id)
+            revision_id = cache_utf8.encode(revision_id)
         inv = Inventory(root_id, revision_id=revision_id)
         for e in elt:
             ie = self._unpack_entry(e)
@@ -240,7 +296,7 @@
         if not InventoryEntry.versionable_kind(kind):
             raise AssertionError('unsupported entry kind %s' % kind)
 
-        get_cached = cache_utf8.get_cached_unicode
+        get_cached = _get_utf8_or_ascii
 
         parent_id = elt.get('parent_id')
         if parent_id is None and not none_parents:
@@ -290,7 +346,7 @@
             if format != '5':
                 raise BzrError("invalid format version %r on inventory"
                                 % format)
-        get_cached = cache_utf8.get_cached_unicode
+        get_cached = _get_utf8_or_ascii
         rev = Revision(committer = elt.get('committer'),
                        timestamp = float(elt.get('timestamp')),
                        revision_id = get_cached(elt.get('revision_id')),

=== modified file 'bzrlib/xml6.py'
--- bzrlib/xml6.py	2006-10-05 05:37:25 +0000
+++ bzrlib/xml6.py	2007-02-10 02:48:43 +0000
@@ -42,7 +42,7 @@
                                                    % format)
         revision_id = elt.get('revision_id')
         if revision_id is not None:
-            revision_id = cache_utf8.get_cached_unicode(revision_id)
+            revision_id = cache_utf8.encode(revision_id)
         inv = inventory.Inventory(root_id=None, revision_id=revision_id)
         for e in elt:
             ie = self._unpack_entry(e, none_parents=True)

